
#pragma once

#include <AK/Optional.h>

namespace Web::CSS {

enum class Keyword;

enum class AlignContent : u8 {
    Normal,
    Start,
    FlexStart,
    End,
    FlexEnd,
    Center,
    SpaceBetween,
    SpaceAround,
    SpaceEvenly,
    Stretch,
};
Optional<AlignContent> keyword_to_align_content(Keyword);
Keyword to_keyword(AlignContent);
StringView to_string(AlignContent);

enum class AlignItems : u8 {
    Baseline,
    Center,
    End,
    FlexEnd,
    FlexStart,
    Normal,
    Safe,
    SelfEnd,
    SelfStart,
    Start,
    Stretch,
    Unsafe,
};
Optional<AlignItems> keyword_to_align_items(Keyword);
Keyword to_keyword(AlignItems);
StringView to_string(AlignItems);

enum class AlignSelf : u8 {
    Auto,
    Baseline,
    Center,
    End,
    FlexEnd,
    FlexStart,
    Normal,
    Safe,
    SelfEnd,
    SelfStart,
    Start,
    Stretch,
    Unsafe,
};
Optional<AlignSelf> keyword_to_align_self(Keyword);
Keyword to_keyword(AlignSelf);
StringView to_string(AlignSelf);

enum class AnimationFillMode : u8 {
    Backwards,
    Both,
    Forwards,
    None,
};
Optional<AnimationFillMode> keyword_to_animation_fill_mode(Keyword);
Keyword to_keyword(AnimationFillMode);
StringView to_string(AnimationFillMode);

enum class AnimationDirection : u8 {
    Alternate,
    AlternateReverse,
    Normal,
    Reverse,
};
Optional<AnimationDirection> keyword_to_animation_direction(Keyword);
Keyword to_keyword(AnimationDirection);
StringView to_string(AnimationDirection);

enum class AnimationPlayState : u8 {
    Paused,
    Running,
};
Optional<AnimationPlayState> keyword_to_animation_play_state(Keyword);
Keyword to_keyword(AnimationPlayState);
StringView to_string(AnimationPlayState);

enum class Appearance : u8 {
    Auto,
    Button,
    Checkbox,
    Listbox,
    Menulist,
    Meter,
    MenulistButton,
    None,
    PushButton,
    ProgressBar,
    Radio,
    Searchfield,
    SliderHorizontal,
    SquareButton,
    Textarea,
    Textfield,
};
Optional<Appearance> keyword_to_appearance(Keyword);
Keyword to_keyword(Appearance);
StringView to_string(Appearance);

enum class BackgroundAttachment : u8 {
    Fixed,
    Local,
    Scroll,
};
Optional<BackgroundAttachment> keyword_to_background_attachment(Keyword);
Keyword to_keyword(BackgroundAttachment);
StringView to_string(BackgroundAttachment);

enum class BackgroundBox : u8 {
    BorderBox,
    ContentBox,
    PaddingBox,
    Text,
};
Optional<BackgroundBox> keyword_to_background_box(Keyword);
Keyword to_keyword(BackgroundBox);
StringView to_string(BackgroundBox);

enum class BorderCollapse : u8 {
    Separate,
    Collapse,
};
Optional<BorderCollapse> keyword_to_border_collapse(Keyword);
Keyword to_keyword(BorderCollapse);
StringView to_string(BorderCollapse);

enum class BoxSizing : u8 {
    BorderBox,
    ContentBox,
};
Optional<BoxSizing> keyword_to_box_sizing(Keyword);
Keyword to_keyword(BoxSizing);
StringView to_string(BoxSizing);

enum class CaptionSide : u8 {
    Top,
    Bottom,
};
Optional<CaptionSide> keyword_to_caption_side(Keyword);
Keyword to_keyword(CaptionSide);
StringView to_string(CaptionSide);

enum class Clear : u8 {
    None,
    Left,
    Right,
    Both,
};
Optional<Clear> keyword_to_clear(Keyword);
Keyword to_keyword(Clear);
StringView to_string(Clear);

enum class ColumnSpan : u8 {
    None,
    All,
};
Optional<ColumnSpan> keyword_to_column_span(Keyword);
Keyword to_keyword(ColumnSpan);
StringView to_string(ColumnSpan);

enum class Contain : u8 {
    None,
    Strict,
    Content,
    Size,
    InlineSize,
    Layout,
    Style,
    Paint,
};
Optional<Contain> keyword_to_contain(Keyword);
Keyword to_keyword(Contain);
StringView to_string(Contain);

enum class ContentVisibility : u8 {
    Visible,
    Auto,
    Hidden,
};
Optional<ContentVisibility> keyword_to_content_visibility(Keyword);
Keyword to_keyword(ContentVisibility);
StringView to_string(ContentVisibility);

enum class CounterStyleNameKeyword : u8 {
    Circle,
    Decimal,
    DecimalLeadingZero,
    Disc,
    DisclosureClosed,
    DisclosureOpen,
    LowerAlpha,
    LowerLatin,
    LowerRoman,
    None,
    Square,
    UpperAlpha,
    UpperLatin,
    UpperRoman,
};
Optional<CounterStyleNameKeyword> keyword_to_counter_style_name_keyword(Keyword);
Keyword to_keyword(CounterStyleNameKeyword);
StringView to_string(CounterStyleNameKeyword);

enum class Cursor : u8 {
    Auto,
    Default,
    None,
    ContextMenu,
    Help,
    Pointer,
    Progress,
    Wait,
    Cell,
    Crosshair,
    Text,
    VerticalText,
    Alias,
    Copy,
    Move,
    NoDrop,
    NotAllowed,
    Grab,
    Grabbing,
    EResize,
    NResize,
    NeResize,
    NwResize,
    SResize,
    SeResize,
    SwResize,
    WResize,
    EwResize,
    NsResize,
    NeswResize,
    NwseResize,
    ColResize,
    RowResize,
    AllScroll,
    ZoomIn,
    ZoomOut,
};
Optional<Cursor> keyword_to_cursor(Keyword);
Keyword to_keyword(Cursor);
StringView to_string(Cursor);

enum class Direction : u8 {
    Ltr,
    Rtl,
};
Optional<Direction> keyword_to_direction(Keyword);
Keyword to_keyword(Direction);
StringView to_string(Direction);

enum class DisplayBox : u8 {
    Contents,
    None,
};
Optional<DisplayBox> keyword_to_display_box(Keyword);
Keyword to_keyword(DisplayBox);
StringView to_string(DisplayBox);

enum class DisplayInside : u8 {
    Flow,
    FlowRoot,
    Table,
    Flex,
    Grid,
    Ruby,
    Math,
};
Optional<DisplayInside> keyword_to_display_inside(Keyword);
Keyword to_keyword(DisplayInside);
StringView to_string(DisplayInside);

enum class DisplayInternal : u8 {
    TableRowGroup,
    TableHeaderGroup,
    TableFooterGroup,
    TableRow,
    TableCell,
    TableColumnGroup,
    TableColumn,
    TableCaption,
    RubyBase,
    RubyText,
    RubyBaseContainer,
    RubyTextContainer,
};
Optional<DisplayInternal> keyword_to_display_internal(Keyword);
Keyword to_keyword(DisplayInternal);
StringView to_string(DisplayInternal);

enum class DisplayLegacy : u8 {
    InlineBlock,
    InlineTable,
    InlineFlex,
    InlineGrid,
};
Optional<DisplayLegacy> keyword_to_display_legacy(Keyword);
Keyword to_keyword(DisplayLegacy);
StringView to_string(DisplayLegacy);

enum class DisplayOutside : u8 {
    Block,
    Inline,
    RunIn,
};
Optional<DisplayOutside> keyword_to_display_outside(Keyword);
Keyword to_keyword(DisplayOutside);
StringView to_string(DisplayOutside);

enum class FillRule : u8 {
    Nonzero,
    Evenodd,
};
Optional<FillRule> keyword_to_fill_rule(Keyword);
Keyword to_keyword(FillRule);
StringView to_string(FillRule);

enum class FlexDirection : u8 {
    Row,
    RowReverse,
    Column,
    ColumnReverse,
};
Optional<FlexDirection> keyword_to_flex_direction(Keyword);
Keyword to_keyword(FlexDirection);
StringView to_string(FlexDirection);

enum class FlexWrap : u8 {
    Nowrap,
    Wrap,
    WrapReverse,
};
Optional<FlexWrap> keyword_to_flex_wrap(Keyword);
Keyword to_keyword(FlexWrap);
StringView to_string(FlexWrap);

enum class Float : u8 {
    None,
    Left,
    Right,
    InlineStart,
    InlineEnd,
};
Optional<Float> keyword_to_float(Keyword);
Keyword to_keyword(Float);
StringView to_string(Float);

enum class FontDisplay : u8 {
    Auto,
    Block,
    Swap,
    Fallback,
    Optional,
};
Optional<FontDisplay> keyword_to_font_display(Keyword);
Keyword to_keyword(FontDisplay);
StringView to_string(FontDisplay);

enum class FontVariantAlternates : u8 {
    Normal,
    HistoricalForms,
};
Optional<FontVariantAlternates> keyword_to_font_variant_alternates(Keyword);
Keyword to_keyword(FontVariantAlternates);
StringView to_string(FontVariantAlternates);

enum class FontVariantCaps : u8 {
    Normal,
    SmallCaps,
    AllSmallCaps,
    PetiteCaps,
    AllPetiteCaps,
    Unicase,
    TitlingCaps,
};
Optional<FontVariantCaps> keyword_to_font_variant_caps(Keyword);
Keyword to_keyword(FontVariantCaps);
StringView to_string(FontVariantCaps);

enum class FontVariantEastAsian : u8 {
    Normal,
    Ruby,
    Jis78,
    Jis83,
    Jis90,
    Jis04,
    Simplified,
    Traditional,
    FullWidth,
    ProportionalWidth,
};
Optional<FontVariantEastAsian> keyword_to_font_variant_east_asian(Keyword);
Keyword to_keyword(FontVariantEastAsian);
StringView to_string(FontVariantEastAsian);

enum class FontVariantEmoji : u8 {
    Normal,
    Text,
    Emoji,
    Unicode,
};
Optional<FontVariantEmoji> keyword_to_font_variant_emoji(Keyword);
Keyword to_keyword(FontVariantEmoji);
StringView to_string(FontVariantEmoji);

enum class FontVariantLigatures : u8 {
    Normal,
    None,
    CommonLigatures,
    NoCommonLigatures,
    DiscretionaryLigatures,
    NoDiscretionaryLigatures,
    HistoricalLigatures,
    NoHistoricalLigatures,
    Contextual,
    NoContextual,
};
Optional<FontVariantLigatures> keyword_to_font_variant_ligatures(Keyword);
Keyword to_keyword(FontVariantLigatures);
StringView to_string(FontVariantLigatures);

enum class FontVariantNumeric : u8 {
    Normal,
    Ordinal,
    SlashedZero,
    LiningNums,
    OldstyleNums,
    ProportionalNums,
    TabularNums,
    DiagonalFractions,
    StackedFractions,
};
Optional<FontVariantNumeric> keyword_to_font_variant_numeric(Keyword);
Keyword to_keyword(FontVariantNumeric);
StringView to_string(FontVariantNumeric);

enum class FontVariantPosition : u8 {
    Normal,
    Sub,
    Super,
};
Optional<FontVariantPosition> keyword_to_font_variant_position(Keyword);
Keyword to_keyword(FontVariantPosition);
StringView to_string(FontVariantPosition);

enum class FontWidth : u8 {
    UltraCondensed,
    ExtraCondensed,
    Condensed,
    SemiCondensed,
    Normal,
    SemiExpanded,
    Expanded,
    ExtraExpanded,
    UltraExpanded,
};
Optional<FontWidth> keyword_to_font_width(Keyword);
Keyword to_keyword(FontWidth);
StringView to_string(FontWidth);

enum class GenericFontFamily : u8 {
    Serif,
    SansSerif,
    Cursive,
    Fantasy,
    Monospace,
    Math,
    UiSerif,
    UiSansSerif,
    UiMonospace,
    UiRounded,
};
Optional<GenericFontFamily> keyword_to_generic_font_family(Keyword);
Keyword to_keyword(GenericFontFamily);
StringView to_string(GenericFontFamily);

enum class ImageRendering : u8 {
    Auto,
    CrispEdges,
    HighQuality,
    Pixelated,
    Smooth,
};
Optional<ImageRendering> keyword_to_image_rendering(Keyword);
Keyword to_keyword(ImageRendering);
StringView to_string(ImageRendering);

enum class Isolation : u8 {
    Auto,
    Isolate,
};
Optional<Isolation> keyword_to_isolation(Keyword);
Keyword to_keyword(Isolation);
StringView to_string(Isolation);

enum class JustifyContent : u8 {
    Normal,
    Start,
    End,
    FlexStart,
    FlexEnd,
    Center,
    SpaceBetween,
    SpaceAround,
    SpaceEvenly,
    Stretch,
    Left,
    Right,
};
Optional<JustifyContent> keyword_to_justify_content(Keyword);
Keyword to_keyword(JustifyContent);
StringView to_string(JustifyContent);

enum class JustifyItems : u8 {
    Baseline,
    Center,
    End,
    FlexEnd,
    FlexStart,
    Legacy,
    Normal,
    Safe,
    SelfEnd,
    SelfStart,
    Start,
    Stretch,
    Unsafe,
    Left,
    Right,
};
Optional<JustifyItems> keyword_to_justify_items(Keyword);
Keyword to_keyword(JustifyItems);
StringView to_string(JustifyItems);

enum class JustifySelf : u8 {
    Auto,
    Baseline,
    Center,
    End,
    FlexEnd,
    FlexStart,
    Normal,
    Safe,
    SelfEnd,
    SelfStart,
    Start,
    Stretch,
    Unsafe,
    Left,
    Right,
};
Optional<JustifySelf> keyword_to_justify_self(Keyword);
Keyword to_keyword(JustifySelf);
StringView to_string(JustifySelf);

enum class LineStyle : u8 {
    None,
    Hidden,
    Dotted,
    Dashed,
    Solid,
    Double,
    Groove,
    Ridge,
    Inset,
    Outset,
};
Optional<LineStyle> keyword_to_line_style(Keyword);
Keyword to_keyword(LineStyle);
StringView to_string(LineStyle);

enum class ListStylePosition : u8 {
    Inside,
    Outside,
};
Optional<ListStylePosition> keyword_to_list_style_position(Keyword);
Keyword to_keyword(ListStylePosition);
StringView to_string(ListStylePosition);

enum class MaskType : u8 {
    Luminance,
    Alpha,
};
Optional<MaskType> keyword_to_mask_type(Keyword);
Keyword to_keyword(MaskType);
StringView to_string(MaskType);

enum class MathShift : u8 {
    Normal,
    Compact,
};
Optional<MathShift> keyword_to_math_shift(Keyword);
Keyword to_keyword(MathShift);
StringView to_string(MathShift);

enum class MathStyle : u8 {
    Normal,
    Compact,
};
Optional<MathStyle> keyword_to_math_style(Keyword);
Keyword to_keyword(MathStyle);
StringView to_string(MathStyle);

enum class MixBlendMode : u8 {
    Normal,
    Multiply,
    Screen,
    Overlay,
    Darken,
    Lighten,
    ColorDodge,
    ColorBurn,
    HardLight,
    SoftLight,
    Difference,
    Exclusion,
    Hue,
    Saturation,
    Color,
    Luminosity,
    PlusDarker,
    PlusLighter,
};
Optional<MixBlendMode> keyword_to_mix_blend_mode(Keyword);
Keyword to_keyword(MixBlendMode);
StringView to_string(MixBlendMode);

enum class ObjectFit : u8 {
    Fill,
    Contain,
    Cover,
    None,
    ScaleDown,
};
Optional<ObjectFit> keyword_to_object_fit(Keyword);
Keyword to_keyword(ObjectFit);
StringView to_string(ObjectFit);

enum class OutlineStyle : u8 {
    Auto,
    None,
    Dotted,
    Dashed,
    Solid,
    Double,
    Groove,
    Ridge,
    Inset,
    Outset,
};
Optional<OutlineStyle> keyword_to_outline_style(Keyword);
Keyword to_keyword(OutlineStyle);
StringView to_string(OutlineStyle);

enum class Overflow : u8 {
    Auto,
    Clip,
    Hidden,
    Scroll,
    Visible,
};
Optional<Overflow> keyword_to_overflow(Keyword);
Keyword to_keyword(Overflow);
StringView to_string(Overflow);

enum class PointerEvents : u8 {
    Auto,
    All,
    None,
};
Optional<PointerEvents> keyword_to_pointer_events(Keyword);
Keyword to_keyword(PointerEvents);
StringView to_string(PointerEvents);

enum class Positioning : u8 {
    Absolute,
    Fixed,
    Relative,
    Static,
    Sticky,
};
Optional<Positioning> keyword_to_positioning(Keyword);
Keyword to_keyword(Positioning);
StringView to_string(Positioning);

enum class PositionEdge : u8 {
    Center,
    Left,
    Right,
    Top,
    Bottom,
};
Optional<PositionEdge> keyword_to_position_edge(Keyword);
Keyword to_keyword(PositionEdge);
StringView to_string(PositionEdge);

enum class Repeat : u8 {
    NoRepeat,
    Repeat,
    Round,
    Space,
};
Optional<Repeat> keyword_to_repeat(Keyword);
Keyword to_keyword(Repeat);
StringView to_string(Repeat);

enum class RoundingStrategy : u8 {
    Down,
    Nearest,
    ToZero,
    Up,
};
Optional<RoundingStrategy> keyword_to_rounding_strategy(Keyword);
Keyword to_keyword(RoundingStrategy);
StringView to_string(RoundingStrategy);

enum class TableLayout : u8 {
    Auto,
    Fixed,
};
Optional<TableLayout> keyword_to_table_layout(Keyword);
Keyword to_keyword(TableLayout);
StringView to_string(TableLayout);

enum class TextAnchor : u8 {
    Start,
    Middle,
    End,
};
Optional<TextAnchor> keyword_to_text_anchor(Keyword);
Keyword to_keyword(TextAnchor);
StringView to_string(TextAnchor);

enum class TextOverflow : u8 {
    Clip,
    Ellipsis,
};
Optional<TextOverflow> keyword_to_text_overflow(Keyword);
Keyword to_keyword(TextOverflow);
StringView to_string(TextOverflow);

enum class ScrollbarGutter : u8 {
    Auto,
    Stable,
    BothEdges,
};
Optional<ScrollbarGutter> keyword_to_scrollbar_gutter(Keyword);
Keyword to_keyword(ScrollbarGutter);
StringView to_string(ScrollbarGutter);

enum class ScrollbarWidth : u8 {
    Auto,
    Thin,
    None,
};
Optional<ScrollbarWidth> keyword_to_scrollbar_width(Keyword);
Keyword to_keyword(ScrollbarWidth);
StringView to_string(ScrollbarWidth);

enum class StrokeLinecap : u8 {
    Butt,
    Square,
    Round,
};
Optional<StrokeLinecap> keyword_to_stroke_linecap(Keyword);
Keyword to_keyword(StrokeLinecap);
StringView to_string(StrokeLinecap);

enum class StrokeLinejoin : u8 {
    Miter,
    Round,
    Bevel,
};
Optional<StrokeLinejoin> keyword_to_stroke_linejoin(Keyword);
Keyword to_keyword(StrokeLinejoin);
StringView to_string(StrokeLinejoin);

enum class TextAlign : u8 {
    Center,
    Justify,
    Start,
    End,
    Left,
    Right,
    MatchParent,
    LibwebCenter,
    LibwebLeft,
    LibwebRight,
};
Optional<TextAlign> keyword_to_text_align(Keyword);
Keyword to_keyword(TextAlign);
StringView to_string(TextAlign);

enum class TextDecorationLine : u8 {
    None,
    Underline,
    Overline,
    LineThrough,
    Blink,
};
Optional<TextDecorationLine> keyword_to_text_decoration_line(Keyword);
Keyword to_keyword(TextDecorationLine);
StringView to_string(TextDecorationLine);

enum class TextDecorationStyle : u8 {
    Dashed,
    Dotted,
    Double,
    Solid,
    Wavy,
};
Optional<TextDecorationStyle> keyword_to_text_decoration_style(Keyword);
Keyword to_keyword(TextDecorationStyle);
StringView to_string(TextDecorationStyle);

enum class TextJustify : u8 {
    Auto,
    None,
    InterWord,
    InterCharacter,
};
Optional<TextJustify> keyword_to_text_justify(Keyword);
Keyword to_keyword(TextJustify);
StringView to_string(TextJustify);

enum class TextTransform : u8 {
    Capitalize,
    FullSizeKana,
    FullWidth,
    Lowercase,
    MathAuto,
    None,
    Uppercase,
};
Optional<TextTransform> keyword_to_text_transform(Keyword);
Keyword to_keyword(TextTransform);
StringView to_string(TextTransform);

enum class TransformBox : u8 {
    ContentBox,
    BorderBox,
    FillBox,
    StrokeBox,
    ViewBox ,
};
Optional<TransformBox> keyword_to_transform_box(Keyword);
Keyword to_keyword(TransformBox);
StringView to_string(TransformBox);

enum class UnicodeBidi : u8 {
    BidiOverride,
    Embed,
    Isolate,
    IsolateOverride,
    Normal,
    Plaintext,
};
Optional<UnicodeBidi> keyword_to_unicode_bidi(Keyword);
Keyword to_keyword(UnicodeBidi);
StringView to_string(UnicodeBidi);

enum class UserSelect : u8 {
    All,
    Auto,
    Contain,
    None,
    Text,
};
Optional<UserSelect> keyword_to_user_select(Keyword);
Keyword to_keyword(UserSelect);
StringView to_string(UserSelect);

enum class VerticalAlign : u8 {
    Baseline,
    Bottom,
    Middle,
    Sub,
    Super,
    TextBottom,
    TextTop,
    Top,
};
Optional<VerticalAlign> keyword_to_vertical_align(Keyword);
Keyword to_keyword(VerticalAlign);
StringView to_string(VerticalAlign);

enum class Visibility : u8 {
    Collapse,
    Hidden,
    Visible,
};
Optional<Visibility> keyword_to_visibility(Keyword);
Keyword to_keyword(Visibility);
StringView to_string(Visibility);

enum class WhiteSpace : u8 {
    Normal,
    Nowrap,
    Pre,
    PreLine,
    PreWrap,
};
Optional<WhiteSpace> keyword_to_white_space(Keyword);
Keyword to_keyword(WhiteSpace);
StringView to_string(WhiteSpace);

enum class WordBreak : u8 {
    Normal,
    KeepAll,
    BreakAll,
    BreakWord,
};
Optional<WordBreak> keyword_to_word_break(Keyword);
Keyword to_keyword(WordBreak);
StringView to_string(WordBreak);

enum class WritingMode : u8 {
    HorizontalTb,
    VerticalRl,
    VerticalLr,
    SidewaysRl,
    SidewaysLr,
};
Optional<WritingMode> keyword_to_writing_mode(Keyword);
Keyword to_keyword(WritingMode);
StringView to_string(WritingMode);

}
