#pragma once
#include <AK/Error.h>
#include <AK/MemoryStream.h>
#include <AK/OwnPtr.h>
#include <AK/Result.h>
#include <AK/Utf8View.h>
#include <LibIPC/Connection.h>
#include <LibIPC/Decoder.h>
#include <LibIPC/Encoder.h>
#include <LibIPC/File.h>
#include <LibIPC/Message.h>
#include <LibIPC/Stub.h>

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdefaulted-function-deleted"
#endif

namespace Messages::WebWorkerClient {

enum class MessageID : i32 {
    DidCloseWorker = 1,
};

class DidCloseWorker final : public IPC::Message {
public:

    DidCloseWorker(DidCloseWorker const&) = default;
    DidCloseWorker(DidCloseWorker&&) = default;
    DidCloseWorker& operator=(DidCloseWorker const&) = default;
    DidCloseWorker() {}

    virtual ~DidCloseWorker() override {}

    virtual u32 endpoint_magic() const override { return 1974781266; }
    virtual i32 message_id() const override { return (int)MessageID::DidCloseWorker; }
    static i32 static_message_id() { return (int)MessageID::DidCloseWorker; }
    virtual const char* message_name() const override { return "WebWorkerClient::DidCloseWorker"; }

    static ErrorOr<NonnullOwnPtr<DidCloseWorker>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<DidCloseWorker>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DidCloseWorker));

        return buffer;
    }

private:

};

} // namespace Messages::WebWorkerClient

template<typename LocalEndpoint, typename PeerEndpoint>
class WebWorkerClientProxy {
public:
    // Used to disambiguate the constructor call.
    struct Tag { };

    WebWorkerClientProxy(IPC::Connection<LocalEndpoint, PeerEndpoint>& connection, Tag)
        : m_connection(connection)
    { }

    void async_did_close_worker(
) {
        MUST(m_connection.post_message(Messages::WebWorkerClient::DidCloseWorker {  }));

    }

private:
    IPC::Connection<LocalEndpoint, PeerEndpoint>& m_connection;
};

template<typename LocalEndpoint, typename PeerEndpoint>
class WebWorkerClientProxy;
class WebWorkerClientStub;

class WebWorkerClientEndpoint {
public:
    template<typename LocalEndpoint>
    using Proxy = WebWorkerClientProxy<LocalEndpoint, WebWorkerClientEndpoint>;
    using Stub = WebWorkerClientStub;

    static u32 static_magic() { return 1974781266; }

    static ErrorOr<NonnullOwnPtr<IPC::Message>> decode_message(ReadonlyBytes buffer, [[maybe_unused]] Queue<IPC::File>& files)
    {
        FixedMemoryStream stream { buffer };
        auto message_endpoint_magic = TRY(stream.read_value<u32>());

        if (message_endpoint_magic != 1974781266) {
            return Error::from_string_literal("Endpoint magic number mismatch, not my message!");
        }

        auto message_id = TRY(stream.read_value<i32>());


        switch (message_id) {

        case (int)Messages::WebWorkerClient::MessageID::DidCloseWorker:
            return TRY(Messages::WebWorkerClient::DidCloseWorker::decode(stream, files));
        default:
            return Error::from_string_literal("Failed to decode WebWorkerClient message");
        }

        VERIFY_NOT_REACHED();
    }

};

class WebWorkerClientStub : public IPC::Stub {
public:
    WebWorkerClientStub() { }
    virtual ~WebWorkerClientStub() override { }

    virtual u32 magic() const override { return 1974781266; }
    virtual ByteString name() const override { return "WebWorkerClient"; }

    virtual ErrorOr<OwnPtr<IPC::MessageBuffer>> handle(const IPC::Message& message) override
    {
        switch (message.message_id()) {

        case (int)Messages::WebWorkerClient::MessageID::DidCloseWorker: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebWorkerClient::DidCloseWorker&>(message);
            did_close_worker();
            return nullptr;

        }

        default:
            return Error::from_string_literal("Unknown message ID for WebWorkerClient endpoint");
        }
    }

    virtual void did_close_worker(
) = 0;

private:
};

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic pop
#endif
