#pragma once
#include <LibURL/URL.h>
#include <LibIPC/File.h>
#include <LibWeb/HTML/StructuredSerialize.h>
#include <LibWeb/HTML/Scripting/SerializedEnvironmentSettingsObject.h>
#include <LibWeb/Bindings/WorkerPrototype.h>
#include <AK/Error.h>
#include <AK/MemoryStream.h>
#include <AK/OwnPtr.h>
#include <AK/Result.h>
#include <AK/Utf8View.h>
#include <LibIPC/Connection.h>
#include <LibIPC/Decoder.h>
#include <LibIPC/Encoder.h>
#include <LibIPC/File.h>
#include <LibIPC/Message.h>
#include <LibIPC/Stub.h>

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdefaulted-function-deleted"
#endif

namespace Messages::WebWorkerServer {

enum class MessageID : i32 {
    StartDedicatedWorker = 1,
    CloseWorker = 2,
    HandleFileReturn = 3,
};

class StartDedicatedWorker final : public IPC::Message {
public:

    StartDedicatedWorker(StartDedicatedWorker const&) = default;
    StartDedicatedWorker(StartDedicatedWorker&&) = default;
    StartDedicatedWorker& operator=(StartDedicatedWorker const&) = default;
    StartDedicatedWorker(URL::URL url, Web::Bindings::WorkerType type, Web::Bindings::RequestCredentials credentials, String name, Web::HTML::TransferDataHolder message_port, Web::HTML::SerializedEnvironmentSettingsObject outside_settings) : m_url(move(url)), m_type(move(type)), m_credentials(move(credentials)), m_name(move(name)), m_message_port(move(message_port)), m_outside_settings(move(outside_settings)) {}

    virtual ~StartDedicatedWorker() override {}

    virtual u32 endpoint_magic() const override { return 3309463393; }
    virtual i32 message_id() const override { return (int)MessageID::StartDedicatedWorker; }
    static i32 static_message_id() { return (int)MessageID::StartDedicatedWorker; }
    virtual const char* message_name() const override { return "WebWorkerServer::StartDedicatedWorker"; }

    static ErrorOr<NonnullOwnPtr<StartDedicatedWorker>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto url = TRY((decoder.decode<URL::URL>()));

        auto type = TRY((decoder.decode<Web::Bindings::WorkerType>()));

        auto credentials = TRY((decoder.decode<Web::Bindings::RequestCredentials>()));

        auto name = TRY((decoder.decode<String>()));

        auto message_port = TRY((decoder.decode<Web::HTML::TransferDataHolder>()));

        auto outside_settings = TRY((decoder.decode<Web::HTML::SerializedEnvironmentSettingsObject>()));

        return make<StartDedicatedWorker>(move(url), move(type), move(credentials), move(name), move(message_port), move(outside_settings));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::StartDedicatedWorker));

        TRY(stream.encode(m_url));

        TRY(stream.encode(m_type));

        TRY(stream.encode(m_credentials));

        TRY(stream.encode(m_name));

        TRY(stream.encode(m_message_port));

        TRY(stream.encode(m_outside_settings));

        return buffer;
    }

    const URL::URL& url() const { return m_url; }
    URL::URL take_url() { return move(m_url); }

    const Web::Bindings::WorkerType& type() const { return m_type; }
    Web::Bindings::WorkerType take_type() { return move(m_type); }

    const Web::Bindings::RequestCredentials& credentials() const { return m_credentials; }
    Web::Bindings::RequestCredentials take_credentials() { return move(m_credentials); }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

    const Web::HTML::TransferDataHolder& message_port() const { return m_message_port; }
    Web::HTML::TransferDataHolder take_message_port() { return move(m_message_port); }

    const Web::HTML::SerializedEnvironmentSettingsObject& outside_settings() const { return m_outside_settings; }
    Web::HTML::SerializedEnvironmentSettingsObject take_outside_settings() { return move(m_outside_settings); }

private:

    URL::URL m_url;

    Web::Bindings::WorkerType m_type;

    Web::Bindings::RequestCredentials m_credentials;

    String m_name;

    Web::HTML::TransferDataHolder m_message_port;

    Web::HTML::SerializedEnvironmentSettingsObject m_outside_settings;

};

class CloseWorker final : public IPC::Message {
public:

    CloseWorker(CloseWorker const&) = default;
    CloseWorker(CloseWorker&&) = default;
    CloseWorker& operator=(CloseWorker const&) = default;
    CloseWorker() {}

    virtual ~CloseWorker() override {}

    virtual u32 endpoint_magic() const override { return 3309463393; }
    virtual i32 message_id() const override { return (int)MessageID::CloseWorker; }
    static i32 static_message_id() { return (int)MessageID::CloseWorker; }
    virtual const char* message_name() const override { return "WebWorkerServer::CloseWorker"; }

    static ErrorOr<NonnullOwnPtr<CloseWorker>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<CloseWorker>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CloseWorker));

        return buffer;
    }

private:

};

class HandleFileReturn final : public IPC::Message {
public:

    HandleFileReturn(HandleFileReturn const&) = default;
    HandleFileReturn(HandleFileReturn&&) = default;
    HandleFileReturn& operator=(HandleFileReturn const&) = default;
    HandleFileReturn(i32 error, Optional<IPC::File> file, i32 request_id) : m_error(move(error)), m_file(move(file)), m_request_id(move(request_id)) {}

    virtual ~HandleFileReturn() override {}

    virtual u32 endpoint_magic() const override { return 3309463393; }
    virtual i32 message_id() const override { return (int)MessageID::HandleFileReturn; }
    static i32 static_message_id() { return (int)MessageID::HandleFileReturn; }
    virtual const char* message_name() const override { return "WebWorkerServer::HandleFileReturn"; }

    static ErrorOr<NonnullOwnPtr<HandleFileReturn>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto error = TRY((decoder.decode<i32>()));

        auto file = TRY((decoder.decode<Optional<IPC::File>>()));

        auto request_id = TRY((decoder.decode<i32>()));

        return make<HandleFileReturn>(move(error), move(file), move(request_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::HandleFileReturn));

        TRY(stream.encode(m_error));

        TRY(stream.encode(m_file));

        TRY(stream.encode(m_request_id));

        return buffer;
    }

    const i32& error() const { return m_error; }
    i32 take_error() { return move(m_error); }

    const Optional<IPC::File>& file() const { return m_file; }
    Optional<IPC::File> take_file() { return move(m_file); }

    const i32& request_id() const { return m_request_id; }
    i32 take_request_id() { return move(m_request_id); }

private:

    i32 m_error;

    Optional<IPC::File> m_file;

    i32 m_request_id;

};

} // namespace Messages::WebWorkerServer

template<typename LocalEndpoint, typename PeerEndpoint>
class WebWorkerServerProxy {
public:
    // Used to disambiguate the constructor call.
    struct Tag { };

    WebWorkerServerProxy(IPC::Connection<LocalEndpoint, PeerEndpoint>& connection, Tag)
        : m_connection(connection)
    { }

    void async_start_dedicated_worker(
URL::URL url, Web::Bindings::WorkerType type, Web::Bindings::RequestCredentials credentials, String name, Web::HTML::TransferDataHolder message_port, Web::HTML::SerializedEnvironmentSettingsObject outside_settings) {
        MUST(m_connection.post_message(Messages::WebWorkerServer::StartDedicatedWorker { move(url), move(type), move(credentials), move(name), move(message_port), move(outside_settings) }));

    }

    void async_close_worker(
) {
        MUST(m_connection.post_message(Messages::WebWorkerServer::CloseWorker {  }));

    }

    void async_handle_file_return(
i32 error, Optional<IPC::File> file, i32 request_id) {
        MUST(m_connection.post_message(Messages::WebWorkerServer::HandleFileReturn { error, move(file), request_id }));

    }

private:
    IPC::Connection<LocalEndpoint, PeerEndpoint>& m_connection;
};

template<typename LocalEndpoint, typename PeerEndpoint>
class WebWorkerServerProxy;
class WebWorkerServerStub;

class WebWorkerServerEndpoint {
public:
    template<typename LocalEndpoint>
    using Proxy = WebWorkerServerProxy<LocalEndpoint, WebWorkerServerEndpoint>;
    using Stub = WebWorkerServerStub;

    static u32 static_magic() { return 3309463393; }

    static ErrorOr<NonnullOwnPtr<IPC::Message>> decode_message(ReadonlyBytes buffer, [[maybe_unused]] Queue<IPC::File>& files)
    {
        FixedMemoryStream stream { buffer };
        auto message_endpoint_magic = TRY(stream.read_value<u32>());

        if (message_endpoint_magic != 3309463393) {
            return Error::from_string_literal("Endpoint magic number mismatch, not my message!");
        }

        auto message_id = TRY(stream.read_value<i32>());


        switch (message_id) {

        case (int)Messages::WebWorkerServer::MessageID::StartDedicatedWorker:
            return TRY(Messages::WebWorkerServer::StartDedicatedWorker::decode(stream, files));
        case (int)Messages::WebWorkerServer::MessageID::CloseWorker:
            return TRY(Messages::WebWorkerServer::CloseWorker::decode(stream, files));
        case (int)Messages::WebWorkerServer::MessageID::HandleFileReturn:
            return TRY(Messages::WebWorkerServer::HandleFileReturn::decode(stream, files));
        default:
            return Error::from_string_literal("Failed to decode WebWorkerServer message");
        }

        VERIFY_NOT_REACHED();
    }

};

class WebWorkerServerStub : public IPC::Stub {
public:
    WebWorkerServerStub() { }
    virtual ~WebWorkerServerStub() override { }

    virtual u32 magic() const override { return 3309463393; }
    virtual ByteString name() const override { return "WebWorkerServer"; }

    virtual ErrorOr<OwnPtr<IPC::MessageBuffer>> handle(const IPC::Message& message) override
    {
        switch (message.message_id()) {

        case (int)Messages::WebWorkerServer::MessageID::StartDedicatedWorker: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebWorkerServer::StartDedicatedWorker&>(message);
            start_dedicated_worker(request.url(), request.type(), request.credentials(), request.name(), request.message_port(), request.outside_settings());
            return nullptr;

        }

        case (int)Messages::WebWorkerServer::MessageID::CloseWorker: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebWorkerServer::CloseWorker&>(message);
            close_worker();
            return nullptr;

        }

        case (int)Messages::WebWorkerServer::MessageID::HandleFileReturn: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebWorkerServer::HandleFileReturn&>(message);
            handle_file_return(request.error(), request.file(), request.request_id());
            return nullptr;

        }

        default:
            return Error::from_string_literal("Unknown message ID for WebWorkerServer endpoint");
        }
    }

    virtual void start_dedicated_worker(
[[maybe_unused]] URL::URL const& url, [[maybe_unused]] Web::Bindings::WorkerType const& type, [[maybe_unused]] Web::Bindings::RequestCredentials const& credentials, [[maybe_unused]] String const& name, [[maybe_unused]] Web::HTML::TransferDataHolder const& message_port, [[maybe_unused]] Web::HTML::SerializedEnvironmentSettingsObject const& outside_settings) = 0;

    virtual void close_worker(
) = 0;

    virtual void handle_file_return(
[[maybe_unused]] i32 error, [[maybe_unused]] Optional<IPC::File> const& file, [[maybe_unused]] i32 request_id) = 0;

private:
};

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic pop
#endif
