#pragma once
#include <LibWeb/WebDriver/Capabilities.h>
#include <LibWeb/WebDriver/Response.h>
#include <LibWeb/WebDriver/UserPrompt.h>
#include <AK/Error.h>
#include <AK/MemoryStream.h>
#include <AK/OwnPtr.h>
#include <AK/Result.h>
#include <AK/Utf8View.h>
#include <LibIPC/Connection.h>
#include <LibIPC/Decoder.h>
#include <LibIPC/Encoder.h>
#include <LibIPC/File.h>
#include <LibIPC/Message.h>
#include <LibIPC/Stub.h>

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wdefaulted-function-deleted"
#endif

namespace Messages::WebDriverClient {

enum class MessageID : i32 {
    CloseSession = 1,
    CloseSessionResponse = 2,
    SetPageLoadStrategy = 3,
    SetUserPromptHandler = 4,
    SetStrictFileInteractability = 5,
    SetIsWebdriverActive = 6,
    GetTimeouts = 7,
    GetTimeoutsResponse = 8,
    SetTimeouts = 9,
    SetTimeoutsResponse = 10,
    NavigateTo = 11,
    NavigateToResponse = 12,
    GetCurrentUrl = 13,
    GetCurrentUrlResponse = 14,
    Back = 15,
    BackResponse = 16,
    Forward = 17,
    ForwardResponse = 18,
    Refresh = 19,
    RefreshResponse = 20,
    GetTitle = 21,
    GetTitleResponse = 22,
    GetWindowHandle = 23,
    GetWindowHandleResponse = 24,
    CloseWindow = 25,
    CloseWindowResponse = 26,
    SwitchToWindow = 27,
    SwitchToWindowResponse = 28,
    NewWindow = 29,
    NewWindowResponse = 30,
    SwitchToFrame = 31,
    SwitchToFrameResponse = 32,
    SwitchToParentFrame = 33,
    SwitchToParentFrameResponse = 34,
    GetWindowRect = 35,
    GetWindowRectResponse = 36,
    SetWindowRect = 37,
    SetWindowRectResponse = 38,
    MaximizeWindow = 39,
    MaximizeWindowResponse = 40,
    MinimizeWindow = 41,
    MinimizeWindowResponse = 42,
    FullscreenWindow = 43,
    FullscreenWindowResponse = 44,
    ConsumeUserActivation = 45,
    ConsumeUserActivationResponse = 46,
    FindElement = 47,
    FindElementResponse = 48,
    FindElements = 49,
    FindElementsResponse = 50,
    FindElementFromElement = 51,
    FindElementFromElementResponse = 52,
    FindElementsFromElement = 53,
    FindElementsFromElementResponse = 54,
    FindElementFromShadowRoot = 55,
    FindElementFromShadowRootResponse = 56,
    FindElementsFromShadowRoot = 57,
    FindElementsFromShadowRootResponse = 58,
    GetActiveElement = 59,
    GetActiveElementResponse = 60,
    GetElementShadowRoot = 61,
    GetElementShadowRootResponse = 62,
    IsElementSelected = 63,
    IsElementSelectedResponse = 64,
    GetElementAttribute = 65,
    GetElementAttributeResponse = 66,
    GetElementProperty = 67,
    GetElementPropertyResponse = 68,
    GetElementCssValue = 69,
    GetElementCssValueResponse = 70,
    GetElementText = 71,
    GetElementTextResponse = 72,
    GetElementTagName = 73,
    GetElementTagNameResponse = 74,
    GetElementRect = 75,
    GetElementRectResponse = 76,
    IsElementEnabled = 77,
    IsElementEnabledResponse = 78,
    GetComputedRole = 79,
    GetComputedRoleResponse = 80,
    GetComputedLabel = 81,
    GetComputedLabelResponse = 82,
    ElementClick = 83,
    ElementClickResponse = 84,
    ElementClear = 85,
    ElementClearResponse = 86,
    ElementSendKeys = 87,
    ElementSendKeysResponse = 88,
    GetSource = 89,
    GetSourceResponse = 90,
    ExecuteScript = 91,
    ExecuteScriptResponse = 92,
    ExecuteAsyncScript = 93,
    ExecuteAsyncScriptResponse = 94,
    GetAllCookies = 95,
    GetAllCookiesResponse = 96,
    GetNamedCookie = 97,
    GetNamedCookieResponse = 98,
    AddCookie = 99,
    AddCookieResponse = 100,
    DeleteCookie = 101,
    DeleteCookieResponse = 102,
    DeleteAllCookies = 103,
    DeleteAllCookiesResponse = 104,
    PerformActions = 105,
    PerformActionsResponse = 106,
    ReleaseActions = 107,
    ReleaseActionsResponse = 108,
    DismissAlert = 109,
    DismissAlertResponse = 110,
    AcceptAlert = 111,
    AcceptAlertResponse = 112,
    GetAlertText = 113,
    GetAlertTextResponse = 114,
    SendAlertText = 115,
    SendAlertTextResponse = 116,
    TakeScreenshot = 117,
    TakeScreenshotResponse = 118,
    TakeElementScreenshot = 119,
    TakeElementScreenshotResponse = 120,
    PrintPage = 121,
    PrintPageResponse = 122,
    EnsureTopLevelBrowsingContextIsOpen = 123,
    EnsureTopLevelBrowsingContextIsOpenResponse = 124,
};

class CloseSessionResponse final : public IPC::Message {
public:

    CloseSessionResponse(CloseSessionResponse const&) = default;
    CloseSessionResponse(CloseSessionResponse&&) = default;
    CloseSessionResponse& operator=(CloseSessionResponse const&) = default;
    CloseSessionResponse() {}

    virtual ~CloseSessionResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::CloseSessionResponse; }
    static i32 static_message_id() { return (int)MessageID::CloseSessionResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::CloseSessionResponse"; }

    static ErrorOr<NonnullOwnPtr<CloseSessionResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<CloseSessionResponse>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CloseSessionResponse));

        return buffer;
    }

private:

};

class CloseSession final : public IPC::Message {
public:

   typedef class CloseSessionResponse ResponseType;

    CloseSession(CloseSession const&) = default;
    CloseSession(CloseSession&&) = default;
    CloseSession& operator=(CloseSession const&) = default;
    CloseSession() {}

    virtual ~CloseSession() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::CloseSession; }
    static i32 static_message_id() { return (int)MessageID::CloseSession; }
    virtual const char* message_name() const override { return "WebDriverClient::CloseSession"; }

    static ErrorOr<NonnullOwnPtr<CloseSession>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<CloseSession>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CloseSession));

        return buffer;
    }

private:

};

class SetPageLoadStrategy final : public IPC::Message {
public:

    SetPageLoadStrategy(SetPageLoadStrategy const&) = default;
    SetPageLoadStrategy(SetPageLoadStrategy&&) = default;
    SetPageLoadStrategy& operator=(SetPageLoadStrategy const&) = default;
    SetPageLoadStrategy(Web::WebDriver::PageLoadStrategy page_load_strategy) : m_page_load_strategy(move(page_load_strategy)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::PageLoadStrategy>)
    SetPageLoadStrategy(WrappedReturnType&& value)
        : m_page_load_strategy(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetPageLoadStrategy() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetPageLoadStrategy; }
    static i32 static_message_id() { return (int)MessageID::SetPageLoadStrategy; }
    virtual const char* message_name() const override { return "WebDriverClient::SetPageLoadStrategy"; }

    static ErrorOr<NonnullOwnPtr<SetPageLoadStrategy>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto page_load_strategy = TRY((decoder.decode<Web::WebDriver::PageLoadStrategy>()));

        return make<SetPageLoadStrategy>(move(page_load_strategy));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetPageLoadStrategy));

        TRY(stream.encode(m_page_load_strategy));

        return buffer;
    }

    const Web::WebDriver::PageLoadStrategy& page_load_strategy() const { return m_page_load_strategy; }
    Web::WebDriver::PageLoadStrategy take_page_load_strategy() { return move(m_page_load_strategy); }

private:

    Web::WebDriver::PageLoadStrategy m_page_load_strategy;

};

class SetUserPromptHandler final : public IPC::Message {
public:

    SetUserPromptHandler(SetUserPromptHandler const&) = default;
    SetUserPromptHandler(SetUserPromptHandler&&) = default;
    SetUserPromptHandler& operator=(SetUserPromptHandler const&) = default;
    SetUserPromptHandler(Web::WebDriver::UserPromptHandler user_prompt_handler) : m_user_prompt_handler(move(user_prompt_handler)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::UserPromptHandler>)
    SetUserPromptHandler(WrappedReturnType&& value)
        : m_user_prompt_handler(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetUserPromptHandler() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetUserPromptHandler; }
    static i32 static_message_id() { return (int)MessageID::SetUserPromptHandler; }
    virtual const char* message_name() const override { return "WebDriverClient::SetUserPromptHandler"; }

    static ErrorOr<NonnullOwnPtr<SetUserPromptHandler>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto user_prompt_handler = TRY((decoder.decode<Web::WebDriver::UserPromptHandler>()));

        return make<SetUserPromptHandler>(move(user_prompt_handler));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetUserPromptHandler));

        TRY(stream.encode(m_user_prompt_handler));

        return buffer;
    }

    const Web::WebDriver::UserPromptHandler& user_prompt_handler() const { return m_user_prompt_handler; }
    Web::WebDriver::UserPromptHandler take_user_prompt_handler() { return move(m_user_prompt_handler); }

private:

    Web::WebDriver::UserPromptHandler m_user_prompt_handler;

};

class SetStrictFileInteractability final : public IPC::Message {
public:

    SetStrictFileInteractability(SetStrictFileInteractability const&) = default;
    SetStrictFileInteractability(SetStrictFileInteractability&&) = default;
    SetStrictFileInteractability& operator=(SetStrictFileInteractability const&) = default;
    SetStrictFileInteractability(bool strict_file_interactability) : m_strict_file_interactability(move(strict_file_interactability)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, bool>)
    SetStrictFileInteractability(WrappedReturnType&& value)
        : m_strict_file_interactability(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetStrictFileInteractability() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetStrictFileInteractability; }
    static i32 static_message_id() { return (int)MessageID::SetStrictFileInteractability; }
    virtual const char* message_name() const override { return "WebDriverClient::SetStrictFileInteractability"; }

    static ErrorOr<NonnullOwnPtr<SetStrictFileInteractability>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto strict_file_interactability = TRY((decoder.decode<bool>()));

        return make<SetStrictFileInteractability>(move(strict_file_interactability));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetStrictFileInteractability));

        TRY(stream.encode(m_strict_file_interactability));

        return buffer;
    }

    const bool& strict_file_interactability() const { return m_strict_file_interactability; }
    bool take_strict_file_interactability() { return move(m_strict_file_interactability); }

private:

    bool m_strict_file_interactability;

};

class SetIsWebdriverActive final : public IPC::Message {
public:

    SetIsWebdriverActive(SetIsWebdriverActive const&) = default;
    SetIsWebdriverActive(SetIsWebdriverActive&&) = default;
    SetIsWebdriverActive& operator=(SetIsWebdriverActive const&) = default;
    SetIsWebdriverActive(bool active) : m_active(move(active)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, bool>)
    SetIsWebdriverActive(WrappedReturnType&& value)
        : m_active(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetIsWebdriverActive() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetIsWebdriverActive; }
    static i32 static_message_id() { return (int)MessageID::SetIsWebdriverActive; }
    virtual const char* message_name() const override { return "WebDriverClient::SetIsWebdriverActive"; }

    static ErrorOr<NonnullOwnPtr<SetIsWebdriverActive>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto active = TRY((decoder.decode<bool>()));

        return make<SetIsWebdriverActive>(move(active));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetIsWebdriverActive));

        TRY(stream.encode(m_active));

        return buffer;
    }

    const bool& active() const { return m_active; }
    bool take_active() { return move(m_active); }

private:

    bool m_active;

};

class GetTimeoutsResponse final : public IPC::Message {
public:

    GetTimeoutsResponse(GetTimeoutsResponse const&) = default;
    GetTimeoutsResponse(GetTimeoutsResponse&&) = default;
    GetTimeoutsResponse& operator=(GetTimeoutsResponse const&) = default;
    GetTimeoutsResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetTimeoutsResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetTimeoutsResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetTimeoutsResponse; }
    static i32 static_message_id() { return (int)MessageID::GetTimeoutsResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetTimeoutsResponse"; }

    static ErrorOr<NonnullOwnPtr<GetTimeoutsResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetTimeoutsResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetTimeoutsResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetTimeouts final : public IPC::Message {
public:

   typedef class GetTimeoutsResponse ResponseType;

    GetTimeouts(GetTimeouts const&) = default;
    GetTimeouts(GetTimeouts&&) = default;
    GetTimeouts& operator=(GetTimeouts const&) = default;
    GetTimeouts() {}

    virtual ~GetTimeouts() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetTimeouts; }
    static i32 static_message_id() { return (int)MessageID::GetTimeouts; }
    virtual const char* message_name() const override { return "WebDriverClient::GetTimeouts"; }

    static ErrorOr<NonnullOwnPtr<GetTimeouts>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetTimeouts>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetTimeouts));

        return buffer;
    }

private:

};

class SetTimeoutsResponse final : public IPC::Message {
public:

    SetTimeoutsResponse(SetTimeoutsResponse const&) = default;
    SetTimeoutsResponse(SetTimeoutsResponse&&) = default;
    SetTimeoutsResponse& operator=(SetTimeoutsResponse const&) = default;
    SetTimeoutsResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SetTimeoutsResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetTimeoutsResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetTimeoutsResponse; }
    static i32 static_message_id() { return (int)MessageID::SetTimeoutsResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SetTimeoutsResponse"; }

    static ErrorOr<NonnullOwnPtr<SetTimeoutsResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SetTimeoutsResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetTimeoutsResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SetTimeouts final : public IPC::Message {
public:

   typedef class SetTimeoutsResponse ResponseType;

    SetTimeouts(SetTimeouts const&) = default;
    SetTimeouts(SetTimeouts&&) = default;
    SetTimeouts& operator=(SetTimeouts const&) = default;
    SetTimeouts(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    SetTimeouts(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetTimeouts() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetTimeouts; }
    static i32 static_message_id() { return (int)MessageID::SetTimeouts; }
    virtual const char* message_name() const override { return "WebDriverClient::SetTimeouts"; }

    static ErrorOr<NonnullOwnPtr<SetTimeouts>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<SetTimeouts>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetTimeouts));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class NavigateToResponse final : public IPC::Message {
public:

    NavigateToResponse(NavigateToResponse const&) = default;
    NavigateToResponse(NavigateToResponse&&) = default;
    NavigateToResponse& operator=(NavigateToResponse const&) = default;
    NavigateToResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    NavigateToResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~NavigateToResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::NavigateToResponse; }
    static i32 static_message_id() { return (int)MessageID::NavigateToResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::NavigateToResponse"; }

    static ErrorOr<NonnullOwnPtr<NavigateToResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<NavigateToResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::NavigateToResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class NavigateTo final : public IPC::Message {
public:

   typedef class NavigateToResponse ResponseType;

    NavigateTo(NavigateTo const&) = default;
    NavigateTo(NavigateTo&&) = default;
    NavigateTo& operator=(NavigateTo const&) = default;
    NavigateTo(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    NavigateTo(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~NavigateTo() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::NavigateTo; }
    static i32 static_message_id() { return (int)MessageID::NavigateTo; }
    virtual const char* message_name() const override { return "WebDriverClient::NavigateTo"; }

    static ErrorOr<NonnullOwnPtr<NavigateTo>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<NavigateTo>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::NavigateTo));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class GetCurrentUrlResponse final : public IPC::Message {
public:

    GetCurrentUrlResponse(GetCurrentUrlResponse const&) = default;
    GetCurrentUrlResponse(GetCurrentUrlResponse&&) = default;
    GetCurrentUrlResponse& operator=(GetCurrentUrlResponse const&) = default;
    GetCurrentUrlResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetCurrentUrlResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetCurrentUrlResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetCurrentUrlResponse; }
    static i32 static_message_id() { return (int)MessageID::GetCurrentUrlResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetCurrentUrlResponse"; }

    static ErrorOr<NonnullOwnPtr<GetCurrentUrlResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetCurrentUrlResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetCurrentUrlResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetCurrentUrl final : public IPC::Message {
public:

   typedef class GetCurrentUrlResponse ResponseType;

    GetCurrentUrl(GetCurrentUrl const&) = default;
    GetCurrentUrl(GetCurrentUrl&&) = default;
    GetCurrentUrl& operator=(GetCurrentUrl const&) = default;
    GetCurrentUrl() {}

    virtual ~GetCurrentUrl() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetCurrentUrl; }
    static i32 static_message_id() { return (int)MessageID::GetCurrentUrl; }
    virtual const char* message_name() const override { return "WebDriverClient::GetCurrentUrl"; }

    static ErrorOr<NonnullOwnPtr<GetCurrentUrl>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetCurrentUrl>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetCurrentUrl));

        return buffer;
    }

private:

};

class BackResponse final : public IPC::Message {
public:

    BackResponse(BackResponse const&) = default;
    BackResponse(BackResponse&&) = default;
    BackResponse& operator=(BackResponse const&) = default;
    BackResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    BackResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~BackResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::BackResponse; }
    static i32 static_message_id() { return (int)MessageID::BackResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::BackResponse"; }

    static ErrorOr<NonnullOwnPtr<BackResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<BackResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::BackResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class Back final : public IPC::Message {
public:

   typedef class BackResponse ResponseType;

    Back(Back const&) = default;
    Back(Back&&) = default;
    Back& operator=(Back const&) = default;
    Back() {}

    virtual ~Back() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::Back; }
    static i32 static_message_id() { return (int)MessageID::Back; }
    virtual const char* message_name() const override { return "WebDriverClient::Back"; }

    static ErrorOr<NonnullOwnPtr<Back>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<Back>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::Back));

        return buffer;
    }

private:

};

class ForwardResponse final : public IPC::Message {
public:

    ForwardResponse(ForwardResponse const&) = default;
    ForwardResponse(ForwardResponse&&) = default;
    ForwardResponse& operator=(ForwardResponse const&) = default;
    ForwardResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ForwardResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ForwardResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ForwardResponse; }
    static i32 static_message_id() { return (int)MessageID::ForwardResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ForwardResponse"; }

    static ErrorOr<NonnullOwnPtr<ForwardResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ForwardResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ForwardResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class Forward final : public IPC::Message {
public:

   typedef class ForwardResponse ResponseType;

    Forward(Forward const&) = default;
    Forward(Forward&&) = default;
    Forward& operator=(Forward const&) = default;
    Forward() {}

    virtual ~Forward() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::Forward; }
    static i32 static_message_id() { return (int)MessageID::Forward; }
    virtual const char* message_name() const override { return "WebDriverClient::Forward"; }

    static ErrorOr<NonnullOwnPtr<Forward>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<Forward>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::Forward));

        return buffer;
    }

private:

};

class RefreshResponse final : public IPC::Message {
public:

    RefreshResponse(RefreshResponse const&) = default;
    RefreshResponse(RefreshResponse&&) = default;
    RefreshResponse& operator=(RefreshResponse const&) = default;
    RefreshResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    RefreshResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~RefreshResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::RefreshResponse; }
    static i32 static_message_id() { return (int)MessageID::RefreshResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::RefreshResponse"; }

    static ErrorOr<NonnullOwnPtr<RefreshResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<RefreshResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::RefreshResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class Refresh final : public IPC::Message {
public:

   typedef class RefreshResponse ResponseType;

    Refresh(Refresh const&) = default;
    Refresh(Refresh&&) = default;
    Refresh& operator=(Refresh const&) = default;
    Refresh() {}

    virtual ~Refresh() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::Refresh; }
    static i32 static_message_id() { return (int)MessageID::Refresh; }
    virtual const char* message_name() const override { return "WebDriverClient::Refresh"; }

    static ErrorOr<NonnullOwnPtr<Refresh>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<Refresh>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::Refresh));

        return buffer;
    }

private:

};

class GetTitleResponse final : public IPC::Message {
public:

    GetTitleResponse(GetTitleResponse const&) = default;
    GetTitleResponse(GetTitleResponse&&) = default;
    GetTitleResponse& operator=(GetTitleResponse const&) = default;
    GetTitleResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetTitleResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetTitleResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetTitleResponse; }
    static i32 static_message_id() { return (int)MessageID::GetTitleResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetTitleResponse"; }

    static ErrorOr<NonnullOwnPtr<GetTitleResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetTitleResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetTitleResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetTitle final : public IPC::Message {
public:

   typedef class GetTitleResponse ResponseType;

    GetTitle(GetTitle const&) = default;
    GetTitle(GetTitle&&) = default;
    GetTitle& operator=(GetTitle const&) = default;
    GetTitle() {}

    virtual ~GetTitle() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetTitle; }
    static i32 static_message_id() { return (int)MessageID::GetTitle; }
    virtual const char* message_name() const override { return "WebDriverClient::GetTitle"; }

    static ErrorOr<NonnullOwnPtr<GetTitle>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetTitle>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetTitle));

        return buffer;
    }

private:

};

class GetWindowHandleResponse final : public IPC::Message {
public:

    GetWindowHandleResponse(GetWindowHandleResponse const&) = default;
    GetWindowHandleResponse(GetWindowHandleResponse&&) = default;
    GetWindowHandleResponse& operator=(GetWindowHandleResponse const&) = default;
    GetWindowHandleResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetWindowHandleResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetWindowHandleResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetWindowHandleResponse; }
    static i32 static_message_id() { return (int)MessageID::GetWindowHandleResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetWindowHandleResponse"; }

    static ErrorOr<NonnullOwnPtr<GetWindowHandleResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetWindowHandleResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetWindowHandleResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetWindowHandle final : public IPC::Message {
public:

   typedef class GetWindowHandleResponse ResponseType;

    GetWindowHandle(GetWindowHandle const&) = default;
    GetWindowHandle(GetWindowHandle&&) = default;
    GetWindowHandle& operator=(GetWindowHandle const&) = default;
    GetWindowHandle() {}

    virtual ~GetWindowHandle() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetWindowHandle; }
    static i32 static_message_id() { return (int)MessageID::GetWindowHandle; }
    virtual const char* message_name() const override { return "WebDriverClient::GetWindowHandle"; }

    static ErrorOr<NonnullOwnPtr<GetWindowHandle>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetWindowHandle>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetWindowHandle));

        return buffer;
    }

private:

};

class CloseWindowResponse final : public IPC::Message {
public:

    CloseWindowResponse(CloseWindowResponse const&) = default;
    CloseWindowResponse(CloseWindowResponse&&) = default;
    CloseWindowResponse& operator=(CloseWindowResponse const&) = default;
    CloseWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    CloseWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~CloseWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::CloseWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::CloseWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::CloseWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<CloseWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<CloseWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CloseWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class CloseWindow final : public IPC::Message {
public:

   typedef class CloseWindowResponse ResponseType;

    CloseWindow(CloseWindow const&) = default;
    CloseWindow(CloseWindow&&) = default;
    CloseWindow& operator=(CloseWindow const&) = default;
    CloseWindow() {}

    virtual ~CloseWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::CloseWindow; }
    static i32 static_message_id() { return (int)MessageID::CloseWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::CloseWindow"; }

    static ErrorOr<NonnullOwnPtr<CloseWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<CloseWindow>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::CloseWindow));

        return buffer;
    }

private:

};

class SwitchToWindowResponse final : public IPC::Message {
public:

    SwitchToWindowResponse(SwitchToWindowResponse const&) = default;
    SwitchToWindowResponse(SwitchToWindowResponse&&) = default;
    SwitchToWindowResponse& operator=(SwitchToWindowResponse const&) = default;
    SwitchToWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SwitchToWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::SwitchToWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<SwitchToWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SwitchToWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SwitchToWindow final : public IPC::Message {
public:

   typedef class SwitchToWindowResponse ResponseType;

    SwitchToWindow(SwitchToWindow const&) = default;
    SwitchToWindow(SwitchToWindow&&) = default;
    SwitchToWindow& operator=(SwitchToWindow const&) = default;
    SwitchToWindow(String handle) : m_handle(move(handle)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    SwitchToWindow(WrappedReturnType&& value)
        : m_handle(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToWindow; }
    static i32 static_message_id() { return (int)MessageID::SwitchToWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToWindow"; }

    static ErrorOr<NonnullOwnPtr<SwitchToWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto handle = TRY((decoder.decode<String>()));

        return make<SwitchToWindow>(move(handle));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToWindow));

        TRY(stream.encode(m_handle));

        return buffer;
    }

    const String& handle() const { return m_handle; }
    String take_handle() { return move(m_handle); }

private:

    String m_handle;

};

class NewWindowResponse final : public IPC::Message {
public:

    NewWindowResponse(NewWindowResponse const&) = default;
    NewWindowResponse(NewWindowResponse&&) = default;
    NewWindowResponse& operator=(NewWindowResponse const&) = default;
    NewWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    NewWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~NewWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::NewWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::NewWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::NewWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<NewWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<NewWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::NewWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class NewWindow final : public IPC::Message {
public:

   typedef class NewWindowResponse ResponseType;

    NewWindow(NewWindow const&) = default;
    NewWindow(NewWindow&&) = default;
    NewWindow& operator=(NewWindow const&) = default;
    NewWindow(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    NewWindow(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~NewWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::NewWindow; }
    static i32 static_message_id() { return (int)MessageID::NewWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::NewWindow"; }

    static ErrorOr<NonnullOwnPtr<NewWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<NewWindow>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::NewWindow));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class SwitchToFrameResponse final : public IPC::Message {
public:

    SwitchToFrameResponse(SwitchToFrameResponse const&) = default;
    SwitchToFrameResponse(SwitchToFrameResponse&&) = default;
    SwitchToFrameResponse& operator=(SwitchToFrameResponse const&) = default;
    SwitchToFrameResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SwitchToFrameResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToFrameResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToFrameResponse; }
    static i32 static_message_id() { return (int)MessageID::SwitchToFrameResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToFrameResponse"; }

    static ErrorOr<NonnullOwnPtr<SwitchToFrameResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SwitchToFrameResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToFrameResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SwitchToFrame final : public IPC::Message {
public:

   typedef class SwitchToFrameResponse ResponseType;

    SwitchToFrame(SwitchToFrame const&) = default;
    SwitchToFrame(SwitchToFrame&&) = default;
    SwitchToFrame& operator=(SwitchToFrame const&) = default;
    SwitchToFrame(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    SwitchToFrame(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToFrame() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToFrame; }
    static i32 static_message_id() { return (int)MessageID::SwitchToFrame; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToFrame"; }

    static ErrorOr<NonnullOwnPtr<SwitchToFrame>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<SwitchToFrame>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToFrame));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class SwitchToParentFrameResponse final : public IPC::Message {
public:

    SwitchToParentFrameResponse(SwitchToParentFrameResponse const&) = default;
    SwitchToParentFrameResponse(SwitchToParentFrameResponse&&) = default;
    SwitchToParentFrameResponse& operator=(SwitchToParentFrameResponse const&) = default;
    SwitchToParentFrameResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SwitchToParentFrameResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToParentFrameResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToParentFrameResponse; }
    static i32 static_message_id() { return (int)MessageID::SwitchToParentFrameResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToParentFrameResponse"; }

    static ErrorOr<NonnullOwnPtr<SwitchToParentFrameResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SwitchToParentFrameResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToParentFrameResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SwitchToParentFrame final : public IPC::Message {
public:

   typedef class SwitchToParentFrameResponse ResponseType;

    SwitchToParentFrame(SwitchToParentFrame const&) = default;
    SwitchToParentFrame(SwitchToParentFrame&&) = default;
    SwitchToParentFrame& operator=(SwitchToParentFrame const&) = default;
    SwitchToParentFrame(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    SwitchToParentFrame(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SwitchToParentFrame() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SwitchToParentFrame; }
    static i32 static_message_id() { return (int)MessageID::SwitchToParentFrame; }
    virtual const char* message_name() const override { return "WebDriverClient::SwitchToParentFrame"; }

    static ErrorOr<NonnullOwnPtr<SwitchToParentFrame>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<SwitchToParentFrame>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SwitchToParentFrame));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class GetWindowRectResponse final : public IPC::Message {
public:

    GetWindowRectResponse(GetWindowRectResponse const&) = default;
    GetWindowRectResponse(GetWindowRectResponse&&) = default;
    GetWindowRectResponse& operator=(GetWindowRectResponse const&) = default;
    GetWindowRectResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetWindowRectResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetWindowRectResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetWindowRectResponse; }
    static i32 static_message_id() { return (int)MessageID::GetWindowRectResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetWindowRectResponse"; }

    static ErrorOr<NonnullOwnPtr<GetWindowRectResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetWindowRectResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetWindowRectResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetWindowRect final : public IPC::Message {
public:

   typedef class GetWindowRectResponse ResponseType;

    GetWindowRect(GetWindowRect const&) = default;
    GetWindowRect(GetWindowRect&&) = default;
    GetWindowRect& operator=(GetWindowRect const&) = default;
    GetWindowRect() {}

    virtual ~GetWindowRect() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetWindowRect; }
    static i32 static_message_id() { return (int)MessageID::GetWindowRect; }
    virtual const char* message_name() const override { return "WebDriverClient::GetWindowRect"; }

    static ErrorOr<NonnullOwnPtr<GetWindowRect>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetWindowRect>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetWindowRect));

        return buffer;
    }

private:

};

class SetWindowRectResponse final : public IPC::Message {
public:

    SetWindowRectResponse(SetWindowRectResponse const&) = default;
    SetWindowRectResponse(SetWindowRectResponse&&) = default;
    SetWindowRectResponse& operator=(SetWindowRectResponse const&) = default;
    SetWindowRectResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SetWindowRectResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetWindowRectResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetWindowRectResponse; }
    static i32 static_message_id() { return (int)MessageID::SetWindowRectResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SetWindowRectResponse"; }

    static ErrorOr<NonnullOwnPtr<SetWindowRectResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SetWindowRectResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetWindowRectResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SetWindowRect final : public IPC::Message {
public:

   typedef class SetWindowRectResponse ResponseType;

    SetWindowRect(SetWindowRect const&) = default;
    SetWindowRect(SetWindowRect&&) = default;
    SetWindowRect& operator=(SetWindowRect const&) = default;
    SetWindowRect(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    SetWindowRect(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SetWindowRect() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SetWindowRect; }
    static i32 static_message_id() { return (int)MessageID::SetWindowRect; }
    virtual const char* message_name() const override { return "WebDriverClient::SetWindowRect"; }

    static ErrorOr<NonnullOwnPtr<SetWindowRect>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<SetWindowRect>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SetWindowRect));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class MaximizeWindowResponse final : public IPC::Message {
public:

    MaximizeWindowResponse(MaximizeWindowResponse const&) = default;
    MaximizeWindowResponse(MaximizeWindowResponse&&) = default;
    MaximizeWindowResponse& operator=(MaximizeWindowResponse const&) = default;
    MaximizeWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    MaximizeWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~MaximizeWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::MaximizeWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::MaximizeWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::MaximizeWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<MaximizeWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<MaximizeWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::MaximizeWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class MaximizeWindow final : public IPC::Message {
public:

   typedef class MaximizeWindowResponse ResponseType;

    MaximizeWindow(MaximizeWindow const&) = default;
    MaximizeWindow(MaximizeWindow&&) = default;
    MaximizeWindow& operator=(MaximizeWindow const&) = default;
    MaximizeWindow() {}

    virtual ~MaximizeWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::MaximizeWindow; }
    static i32 static_message_id() { return (int)MessageID::MaximizeWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::MaximizeWindow"; }

    static ErrorOr<NonnullOwnPtr<MaximizeWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<MaximizeWindow>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::MaximizeWindow));

        return buffer;
    }

private:

};

class MinimizeWindowResponse final : public IPC::Message {
public:

    MinimizeWindowResponse(MinimizeWindowResponse const&) = default;
    MinimizeWindowResponse(MinimizeWindowResponse&&) = default;
    MinimizeWindowResponse& operator=(MinimizeWindowResponse const&) = default;
    MinimizeWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    MinimizeWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~MinimizeWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::MinimizeWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::MinimizeWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::MinimizeWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<MinimizeWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<MinimizeWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::MinimizeWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class MinimizeWindow final : public IPC::Message {
public:

   typedef class MinimizeWindowResponse ResponseType;

    MinimizeWindow(MinimizeWindow const&) = default;
    MinimizeWindow(MinimizeWindow&&) = default;
    MinimizeWindow& operator=(MinimizeWindow const&) = default;
    MinimizeWindow() {}

    virtual ~MinimizeWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::MinimizeWindow; }
    static i32 static_message_id() { return (int)MessageID::MinimizeWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::MinimizeWindow"; }

    static ErrorOr<NonnullOwnPtr<MinimizeWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<MinimizeWindow>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::MinimizeWindow));

        return buffer;
    }

private:

};

class FullscreenWindowResponse final : public IPC::Message {
public:

    FullscreenWindowResponse(FullscreenWindowResponse const&) = default;
    FullscreenWindowResponse(FullscreenWindowResponse&&) = default;
    FullscreenWindowResponse& operator=(FullscreenWindowResponse const&) = default;
    FullscreenWindowResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FullscreenWindowResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FullscreenWindowResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FullscreenWindowResponse; }
    static i32 static_message_id() { return (int)MessageID::FullscreenWindowResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FullscreenWindowResponse"; }

    static ErrorOr<NonnullOwnPtr<FullscreenWindowResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FullscreenWindowResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FullscreenWindowResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FullscreenWindow final : public IPC::Message {
public:

   typedef class FullscreenWindowResponse ResponseType;

    FullscreenWindow(FullscreenWindow const&) = default;
    FullscreenWindow(FullscreenWindow&&) = default;
    FullscreenWindow& operator=(FullscreenWindow const&) = default;
    FullscreenWindow() {}

    virtual ~FullscreenWindow() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FullscreenWindow; }
    static i32 static_message_id() { return (int)MessageID::FullscreenWindow; }
    virtual const char* message_name() const override { return "WebDriverClient::FullscreenWindow"; }

    static ErrorOr<NonnullOwnPtr<FullscreenWindow>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<FullscreenWindow>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FullscreenWindow));

        return buffer;
    }

private:

};

class ConsumeUserActivationResponse final : public IPC::Message {
public:

    ConsumeUserActivationResponse(ConsumeUserActivationResponse const&) = default;
    ConsumeUserActivationResponse(ConsumeUserActivationResponse&&) = default;
    ConsumeUserActivationResponse& operator=(ConsumeUserActivationResponse const&) = default;
    ConsumeUserActivationResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ConsumeUserActivationResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ConsumeUserActivationResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ConsumeUserActivationResponse; }
    static i32 static_message_id() { return (int)MessageID::ConsumeUserActivationResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ConsumeUserActivationResponse"; }

    static ErrorOr<NonnullOwnPtr<ConsumeUserActivationResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ConsumeUserActivationResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ConsumeUserActivationResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ConsumeUserActivation final : public IPC::Message {
public:

   typedef class ConsumeUserActivationResponse ResponseType;

    ConsumeUserActivation(ConsumeUserActivation const&) = default;
    ConsumeUserActivation(ConsumeUserActivation&&) = default;
    ConsumeUserActivation& operator=(ConsumeUserActivation const&) = default;
    ConsumeUserActivation() {}

    virtual ~ConsumeUserActivation() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ConsumeUserActivation; }
    static i32 static_message_id() { return (int)MessageID::ConsumeUserActivation; }
    virtual const char* message_name() const override { return "WebDriverClient::ConsumeUserActivation"; }

    static ErrorOr<NonnullOwnPtr<ConsumeUserActivation>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<ConsumeUserActivation>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ConsumeUserActivation));

        return buffer;
    }

private:

};

class FindElementResponse final : public IPC::Message {
public:

    FindElementResponse(FindElementResponse const&) = default;
    FindElementResponse(FindElementResponse&&) = default;
    FindElementResponse& operator=(FindElementResponse const&) = default;
    FindElementResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElement final : public IPC::Message {
public:

   typedef class FindElementResponse ResponseType;

    FindElement(FindElement const&) = default;
    FindElement(FindElement&&) = default;
    FindElement& operator=(FindElement const&) = default;
    FindElement(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    FindElement(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElement() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElement; }
    static i32 static_message_id() { return (int)MessageID::FindElement; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElement"; }

    static ErrorOr<NonnullOwnPtr<FindElement>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<FindElement>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElement));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class FindElementsResponse final : public IPC::Message {
public:

    FindElementsResponse(FindElementsResponse const&) = default;
    FindElementsResponse(FindElementsResponse&&) = default;
    FindElementsResponse& operator=(FindElementsResponse const&) = default;
    FindElementsResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementsResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementsResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementsResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementsResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementsResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementsResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementsResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementsResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElements final : public IPC::Message {
public:

   typedef class FindElementsResponse ResponseType;

    FindElements(FindElements const&) = default;
    FindElements(FindElements&&) = default;
    FindElements& operator=(FindElements const&) = default;
    FindElements(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    FindElements(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElements() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElements; }
    static i32 static_message_id() { return (int)MessageID::FindElements; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElements"; }

    static ErrorOr<NonnullOwnPtr<FindElements>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<FindElements>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElements));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class FindElementFromElementResponse final : public IPC::Message {
public:

    FindElementFromElementResponse(FindElementFromElementResponse const&) = default;
    FindElementFromElementResponse(FindElementFromElementResponse&&) = default;
    FindElementFromElementResponse& operator=(FindElementFromElementResponse const&) = default;
    FindElementFromElementResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementFromElementResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementFromElementResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementFromElementResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementFromElementResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementFromElementResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementFromElementResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementFromElementResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementFromElementResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElementFromElement final : public IPC::Message {
public:

   typedef class FindElementFromElementResponse ResponseType;

    FindElementFromElement(FindElementFromElement const&) = default;
    FindElementFromElement(FindElementFromElement&&) = default;
    FindElementFromElement& operator=(FindElementFromElement const&) = default;
    FindElementFromElement(JsonValue payload, String element_id) : m_payload(move(payload)), m_element_id(move(element_id)) {}

    virtual ~FindElementFromElement() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementFromElement; }
    static i32 static_message_id() { return (int)MessageID::FindElementFromElement; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementFromElement"; }

    static ErrorOr<NonnullOwnPtr<FindElementFromElement>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        auto element_id = TRY((decoder.decode<String>()));

        return make<FindElementFromElement>(move(payload), move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementFromElement));

        TRY(stream.encode(m_payload));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    JsonValue m_payload;

    String m_element_id;

};

class FindElementsFromElementResponse final : public IPC::Message {
public:

    FindElementsFromElementResponse(FindElementsFromElementResponse const&) = default;
    FindElementsFromElementResponse(FindElementsFromElementResponse&&) = default;
    FindElementsFromElementResponse& operator=(FindElementsFromElementResponse const&) = default;
    FindElementsFromElementResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementsFromElementResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementsFromElementResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementsFromElementResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementsFromElementResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementsFromElementResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementsFromElementResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementsFromElementResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementsFromElementResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElementsFromElement final : public IPC::Message {
public:

   typedef class FindElementsFromElementResponse ResponseType;

    FindElementsFromElement(FindElementsFromElement const&) = default;
    FindElementsFromElement(FindElementsFromElement&&) = default;
    FindElementsFromElement& operator=(FindElementsFromElement const&) = default;
    FindElementsFromElement(JsonValue payload, String element_id) : m_payload(move(payload)), m_element_id(move(element_id)) {}

    virtual ~FindElementsFromElement() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementsFromElement; }
    static i32 static_message_id() { return (int)MessageID::FindElementsFromElement; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementsFromElement"; }

    static ErrorOr<NonnullOwnPtr<FindElementsFromElement>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        auto element_id = TRY((decoder.decode<String>()));

        return make<FindElementsFromElement>(move(payload), move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementsFromElement));

        TRY(stream.encode(m_payload));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    JsonValue m_payload;

    String m_element_id;

};

class FindElementFromShadowRootResponse final : public IPC::Message {
public:

    FindElementFromShadowRootResponse(FindElementFromShadowRootResponse const&) = default;
    FindElementFromShadowRootResponse(FindElementFromShadowRootResponse&&) = default;
    FindElementFromShadowRootResponse& operator=(FindElementFromShadowRootResponse const&) = default;
    FindElementFromShadowRootResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementFromShadowRootResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementFromShadowRootResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementFromShadowRootResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementFromShadowRootResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementFromShadowRootResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementFromShadowRootResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementFromShadowRootResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementFromShadowRootResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElementFromShadowRoot final : public IPC::Message {
public:

   typedef class FindElementFromShadowRootResponse ResponseType;

    FindElementFromShadowRoot(FindElementFromShadowRoot const&) = default;
    FindElementFromShadowRoot(FindElementFromShadowRoot&&) = default;
    FindElementFromShadowRoot& operator=(FindElementFromShadowRoot const&) = default;
    FindElementFromShadowRoot(JsonValue payload, String shadow_id) : m_payload(move(payload)), m_shadow_id(move(shadow_id)) {}

    virtual ~FindElementFromShadowRoot() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementFromShadowRoot; }
    static i32 static_message_id() { return (int)MessageID::FindElementFromShadowRoot; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementFromShadowRoot"; }

    static ErrorOr<NonnullOwnPtr<FindElementFromShadowRoot>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        auto shadow_id = TRY((decoder.decode<String>()));

        return make<FindElementFromShadowRoot>(move(payload), move(shadow_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementFromShadowRoot));

        TRY(stream.encode(m_payload));

        TRY(stream.encode(m_shadow_id));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

    const String& shadow_id() const { return m_shadow_id; }
    String take_shadow_id() { return move(m_shadow_id); }

private:

    JsonValue m_payload;

    String m_shadow_id;

};

class FindElementsFromShadowRootResponse final : public IPC::Message {
public:

    FindElementsFromShadowRootResponse(FindElementsFromShadowRootResponse const&) = default;
    FindElementsFromShadowRootResponse(FindElementsFromShadowRootResponse&&) = default;
    FindElementsFromShadowRootResponse& operator=(FindElementsFromShadowRootResponse const&) = default;
    FindElementsFromShadowRootResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    FindElementsFromShadowRootResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~FindElementsFromShadowRootResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementsFromShadowRootResponse; }
    static i32 static_message_id() { return (int)MessageID::FindElementsFromShadowRootResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementsFromShadowRootResponse"; }

    static ErrorOr<NonnullOwnPtr<FindElementsFromShadowRootResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<FindElementsFromShadowRootResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementsFromShadowRootResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class FindElementsFromShadowRoot final : public IPC::Message {
public:

   typedef class FindElementsFromShadowRootResponse ResponseType;

    FindElementsFromShadowRoot(FindElementsFromShadowRoot const&) = default;
    FindElementsFromShadowRoot(FindElementsFromShadowRoot&&) = default;
    FindElementsFromShadowRoot& operator=(FindElementsFromShadowRoot const&) = default;
    FindElementsFromShadowRoot(JsonValue payload, String shadow_id) : m_payload(move(payload)), m_shadow_id(move(shadow_id)) {}

    virtual ~FindElementsFromShadowRoot() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::FindElementsFromShadowRoot; }
    static i32 static_message_id() { return (int)MessageID::FindElementsFromShadowRoot; }
    virtual const char* message_name() const override { return "WebDriverClient::FindElementsFromShadowRoot"; }

    static ErrorOr<NonnullOwnPtr<FindElementsFromShadowRoot>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        auto shadow_id = TRY((decoder.decode<String>()));

        return make<FindElementsFromShadowRoot>(move(payload), move(shadow_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::FindElementsFromShadowRoot));

        TRY(stream.encode(m_payload));

        TRY(stream.encode(m_shadow_id));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

    const String& shadow_id() const { return m_shadow_id; }
    String take_shadow_id() { return move(m_shadow_id); }

private:

    JsonValue m_payload;

    String m_shadow_id;

};

class GetActiveElementResponse final : public IPC::Message {
public:

    GetActiveElementResponse(GetActiveElementResponse const&) = default;
    GetActiveElementResponse(GetActiveElementResponse&&) = default;
    GetActiveElementResponse& operator=(GetActiveElementResponse const&) = default;
    GetActiveElementResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetActiveElementResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetActiveElementResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetActiveElementResponse; }
    static i32 static_message_id() { return (int)MessageID::GetActiveElementResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetActiveElementResponse"; }

    static ErrorOr<NonnullOwnPtr<GetActiveElementResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetActiveElementResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetActiveElementResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetActiveElement final : public IPC::Message {
public:

   typedef class GetActiveElementResponse ResponseType;

    GetActiveElement(GetActiveElement const&) = default;
    GetActiveElement(GetActiveElement&&) = default;
    GetActiveElement& operator=(GetActiveElement const&) = default;
    GetActiveElement() {}

    virtual ~GetActiveElement() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetActiveElement; }
    static i32 static_message_id() { return (int)MessageID::GetActiveElement; }
    virtual const char* message_name() const override { return "WebDriverClient::GetActiveElement"; }

    static ErrorOr<NonnullOwnPtr<GetActiveElement>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetActiveElement>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetActiveElement));

        return buffer;
    }

private:

};

class GetElementShadowRootResponse final : public IPC::Message {
public:

    GetElementShadowRootResponse(GetElementShadowRootResponse const&) = default;
    GetElementShadowRootResponse(GetElementShadowRootResponse&&) = default;
    GetElementShadowRootResponse& operator=(GetElementShadowRootResponse const&) = default;
    GetElementShadowRootResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementShadowRootResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementShadowRootResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementShadowRootResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementShadowRootResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementShadowRootResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementShadowRootResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementShadowRootResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementShadowRootResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementShadowRoot final : public IPC::Message {
public:

   typedef class GetElementShadowRootResponse ResponseType;

    GetElementShadowRoot(GetElementShadowRoot const&) = default;
    GetElementShadowRoot(GetElementShadowRoot&&) = default;
    GetElementShadowRoot& operator=(GetElementShadowRoot const&) = default;
    GetElementShadowRoot(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetElementShadowRoot(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementShadowRoot() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementShadowRoot; }
    static i32 static_message_id() { return (int)MessageID::GetElementShadowRoot; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementShadowRoot"; }

    static ErrorOr<NonnullOwnPtr<GetElementShadowRoot>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetElementShadowRoot>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementShadowRoot));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class IsElementSelectedResponse final : public IPC::Message {
public:

    IsElementSelectedResponse(IsElementSelectedResponse const&) = default;
    IsElementSelectedResponse(IsElementSelectedResponse&&) = default;
    IsElementSelectedResponse& operator=(IsElementSelectedResponse const&) = default;
    IsElementSelectedResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    IsElementSelectedResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~IsElementSelectedResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::IsElementSelectedResponse; }
    static i32 static_message_id() { return (int)MessageID::IsElementSelectedResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::IsElementSelectedResponse"; }

    static ErrorOr<NonnullOwnPtr<IsElementSelectedResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<IsElementSelectedResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::IsElementSelectedResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class IsElementSelected final : public IPC::Message {
public:

   typedef class IsElementSelectedResponse ResponseType;

    IsElementSelected(IsElementSelected const&) = default;
    IsElementSelected(IsElementSelected&&) = default;
    IsElementSelected& operator=(IsElementSelected const&) = default;
    IsElementSelected(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    IsElementSelected(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~IsElementSelected() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::IsElementSelected; }
    static i32 static_message_id() { return (int)MessageID::IsElementSelected; }
    virtual const char* message_name() const override { return "WebDriverClient::IsElementSelected"; }

    static ErrorOr<NonnullOwnPtr<IsElementSelected>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<IsElementSelected>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::IsElementSelected));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class GetElementAttributeResponse final : public IPC::Message {
public:

    GetElementAttributeResponse(GetElementAttributeResponse const&) = default;
    GetElementAttributeResponse(GetElementAttributeResponse&&) = default;
    GetElementAttributeResponse& operator=(GetElementAttributeResponse const&) = default;
    GetElementAttributeResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementAttributeResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementAttributeResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementAttributeResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementAttributeResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementAttributeResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementAttributeResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementAttributeResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementAttributeResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementAttribute final : public IPC::Message {
public:

   typedef class GetElementAttributeResponse ResponseType;

    GetElementAttribute(GetElementAttribute const&) = default;
    GetElementAttribute(GetElementAttribute&&) = default;
    GetElementAttribute& operator=(GetElementAttribute const&) = default;
    GetElementAttribute(String element_id, String name) : m_element_id(move(element_id)), m_name(move(name)) {}

    virtual ~GetElementAttribute() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementAttribute; }
    static i32 static_message_id() { return (int)MessageID::GetElementAttribute; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementAttribute"; }

    static ErrorOr<NonnullOwnPtr<GetElementAttribute>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        auto name = TRY((decoder.decode<String>()));

        return make<GetElementAttribute>(move(element_id), move(name));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementAttribute));

        TRY(stream.encode(m_element_id));

        TRY(stream.encode(m_name));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

private:

    String m_element_id;

    String m_name;

};

class GetElementPropertyResponse final : public IPC::Message {
public:

    GetElementPropertyResponse(GetElementPropertyResponse const&) = default;
    GetElementPropertyResponse(GetElementPropertyResponse&&) = default;
    GetElementPropertyResponse& operator=(GetElementPropertyResponse const&) = default;
    GetElementPropertyResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementPropertyResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementPropertyResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementPropertyResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementPropertyResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementPropertyResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementPropertyResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementPropertyResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementPropertyResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementProperty final : public IPC::Message {
public:

   typedef class GetElementPropertyResponse ResponseType;

    GetElementProperty(GetElementProperty const&) = default;
    GetElementProperty(GetElementProperty&&) = default;
    GetElementProperty& operator=(GetElementProperty const&) = default;
    GetElementProperty(String element_id, String name) : m_element_id(move(element_id)), m_name(move(name)) {}

    virtual ~GetElementProperty() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementProperty; }
    static i32 static_message_id() { return (int)MessageID::GetElementProperty; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementProperty"; }

    static ErrorOr<NonnullOwnPtr<GetElementProperty>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        auto name = TRY((decoder.decode<String>()));

        return make<GetElementProperty>(move(element_id), move(name));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementProperty));

        TRY(stream.encode(m_element_id));

        TRY(stream.encode(m_name));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

private:

    String m_element_id;

    String m_name;

};

class GetElementCssValueResponse final : public IPC::Message {
public:

    GetElementCssValueResponse(GetElementCssValueResponse const&) = default;
    GetElementCssValueResponse(GetElementCssValueResponse&&) = default;
    GetElementCssValueResponse& operator=(GetElementCssValueResponse const&) = default;
    GetElementCssValueResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementCssValueResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementCssValueResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementCssValueResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementCssValueResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementCssValueResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementCssValueResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementCssValueResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementCssValueResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementCssValue final : public IPC::Message {
public:

   typedef class GetElementCssValueResponse ResponseType;

    GetElementCssValue(GetElementCssValue const&) = default;
    GetElementCssValue(GetElementCssValue&&) = default;
    GetElementCssValue& operator=(GetElementCssValue const&) = default;
    GetElementCssValue(String element_id, String name) : m_element_id(move(element_id)), m_name(move(name)) {}

    virtual ~GetElementCssValue() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementCssValue; }
    static i32 static_message_id() { return (int)MessageID::GetElementCssValue; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementCssValue"; }

    static ErrorOr<NonnullOwnPtr<GetElementCssValue>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        auto name = TRY((decoder.decode<String>()));

        return make<GetElementCssValue>(move(element_id), move(name));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementCssValue));

        TRY(stream.encode(m_element_id));

        TRY(stream.encode(m_name));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

private:

    String m_element_id;

    String m_name;

};

class GetElementTextResponse final : public IPC::Message {
public:

    GetElementTextResponse(GetElementTextResponse const&) = default;
    GetElementTextResponse(GetElementTextResponse&&) = default;
    GetElementTextResponse& operator=(GetElementTextResponse const&) = default;
    GetElementTextResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementTextResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementTextResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementTextResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementTextResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementTextResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementTextResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementTextResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementTextResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementText final : public IPC::Message {
public:

   typedef class GetElementTextResponse ResponseType;

    GetElementText(GetElementText const&) = default;
    GetElementText(GetElementText&&) = default;
    GetElementText& operator=(GetElementText const&) = default;
    GetElementText(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetElementText(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementText() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementText; }
    static i32 static_message_id() { return (int)MessageID::GetElementText; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementText"; }

    static ErrorOr<NonnullOwnPtr<GetElementText>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetElementText>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementText));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class GetElementTagNameResponse final : public IPC::Message {
public:

    GetElementTagNameResponse(GetElementTagNameResponse const&) = default;
    GetElementTagNameResponse(GetElementTagNameResponse&&) = default;
    GetElementTagNameResponse& operator=(GetElementTagNameResponse const&) = default;
    GetElementTagNameResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementTagNameResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementTagNameResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementTagNameResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementTagNameResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementTagNameResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementTagNameResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementTagNameResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementTagNameResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementTagName final : public IPC::Message {
public:

   typedef class GetElementTagNameResponse ResponseType;

    GetElementTagName(GetElementTagName const&) = default;
    GetElementTagName(GetElementTagName&&) = default;
    GetElementTagName& operator=(GetElementTagName const&) = default;
    GetElementTagName(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetElementTagName(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementTagName() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementTagName; }
    static i32 static_message_id() { return (int)MessageID::GetElementTagName; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementTagName"; }

    static ErrorOr<NonnullOwnPtr<GetElementTagName>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetElementTagName>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementTagName));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class GetElementRectResponse final : public IPC::Message {
public:

    GetElementRectResponse(GetElementRectResponse const&) = default;
    GetElementRectResponse(GetElementRectResponse&&) = default;
    GetElementRectResponse& operator=(GetElementRectResponse const&) = default;
    GetElementRectResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetElementRectResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementRectResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementRectResponse; }
    static i32 static_message_id() { return (int)MessageID::GetElementRectResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementRectResponse"; }

    static ErrorOr<NonnullOwnPtr<GetElementRectResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetElementRectResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementRectResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetElementRect final : public IPC::Message {
public:

   typedef class GetElementRectResponse ResponseType;

    GetElementRect(GetElementRect const&) = default;
    GetElementRect(GetElementRect&&) = default;
    GetElementRect& operator=(GetElementRect const&) = default;
    GetElementRect(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetElementRect(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetElementRect() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetElementRect; }
    static i32 static_message_id() { return (int)MessageID::GetElementRect; }
    virtual const char* message_name() const override { return "WebDriverClient::GetElementRect"; }

    static ErrorOr<NonnullOwnPtr<GetElementRect>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetElementRect>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetElementRect));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class IsElementEnabledResponse final : public IPC::Message {
public:

    IsElementEnabledResponse(IsElementEnabledResponse const&) = default;
    IsElementEnabledResponse(IsElementEnabledResponse&&) = default;
    IsElementEnabledResponse& operator=(IsElementEnabledResponse const&) = default;
    IsElementEnabledResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    IsElementEnabledResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~IsElementEnabledResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::IsElementEnabledResponse; }
    static i32 static_message_id() { return (int)MessageID::IsElementEnabledResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::IsElementEnabledResponse"; }

    static ErrorOr<NonnullOwnPtr<IsElementEnabledResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<IsElementEnabledResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::IsElementEnabledResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class IsElementEnabled final : public IPC::Message {
public:

   typedef class IsElementEnabledResponse ResponseType;

    IsElementEnabled(IsElementEnabled const&) = default;
    IsElementEnabled(IsElementEnabled&&) = default;
    IsElementEnabled& operator=(IsElementEnabled const&) = default;
    IsElementEnabled(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    IsElementEnabled(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~IsElementEnabled() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::IsElementEnabled; }
    static i32 static_message_id() { return (int)MessageID::IsElementEnabled; }
    virtual const char* message_name() const override { return "WebDriverClient::IsElementEnabled"; }

    static ErrorOr<NonnullOwnPtr<IsElementEnabled>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<IsElementEnabled>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::IsElementEnabled));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class GetComputedRoleResponse final : public IPC::Message {
public:

    GetComputedRoleResponse(GetComputedRoleResponse const&) = default;
    GetComputedRoleResponse(GetComputedRoleResponse&&) = default;
    GetComputedRoleResponse& operator=(GetComputedRoleResponse const&) = default;
    GetComputedRoleResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetComputedRoleResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetComputedRoleResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetComputedRoleResponse; }
    static i32 static_message_id() { return (int)MessageID::GetComputedRoleResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetComputedRoleResponse"; }

    static ErrorOr<NonnullOwnPtr<GetComputedRoleResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetComputedRoleResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetComputedRoleResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetComputedRole final : public IPC::Message {
public:

   typedef class GetComputedRoleResponse ResponseType;

    GetComputedRole(GetComputedRole const&) = default;
    GetComputedRole(GetComputedRole&&) = default;
    GetComputedRole& operator=(GetComputedRole const&) = default;
    GetComputedRole(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetComputedRole(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetComputedRole() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetComputedRole; }
    static i32 static_message_id() { return (int)MessageID::GetComputedRole; }
    virtual const char* message_name() const override { return "WebDriverClient::GetComputedRole"; }

    static ErrorOr<NonnullOwnPtr<GetComputedRole>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetComputedRole>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetComputedRole));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class GetComputedLabelResponse final : public IPC::Message {
public:

    GetComputedLabelResponse(GetComputedLabelResponse const&) = default;
    GetComputedLabelResponse(GetComputedLabelResponse&&) = default;
    GetComputedLabelResponse& operator=(GetComputedLabelResponse const&) = default;
    GetComputedLabelResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetComputedLabelResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetComputedLabelResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetComputedLabelResponse; }
    static i32 static_message_id() { return (int)MessageID::GetComputedLabelResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetComputedLabelResponse"; }

    static ErrorOr<NonnullOwnPtr<GetComputedLabelResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetComputedLabelResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetComputedLabelResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetComputedLabel final : public IPC::Message {
public:

   typedef class GetComputedLabelResponse ResponseType;

    GetComputedLabel(GetComputedLabel const&) = default;
    GetComputedLabel(GetComputedLabel&&) = default;
    GetComputedLabel& operator=(GetComputedLabel const&) = default;
    GetComputedLabel(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetComputedLabel(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetComputedLabel() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetComputedLabel; }
    static i32 static_message_id() { return (int)MessageID::GetComputedLabel; }
    virtual const char* message_name() const override { return "WebDriverClient::GetComputedLabel"; }

    static ErrorOr<NonnullOwnPtr<GetComputedLabel>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<GetComputedLabel>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetComputedLabel));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class ElementClickResponse final : public IPC::Message {
public:

    ElementClickResponse(ElementClickResponse const&) = default;
    ElementClickResponse(ElementClickResponse&&) = default;
    ElementClickResponse& operator=(ElementClickResponse const&) = default;
    ElementClickResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ElementClickResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ElementClickResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementClickResponse; }
    static i32 static_message_id() { return (int)MessageID::ElementClickResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementClickResponse"; }

    static ErrorOr<NonnullOwnPtr<ElementClickResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ElementClickResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementClickResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ElementClick final : public IPC::Message {
public:

   typedef class ElementClickResponse ResponseType;

    ElementClick(ElementClick const&) = default;
    ElementClick(ElementClick&&) = default;
    ElementClick& operator=(ElementClick const&) = default;
    ElementClick(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    ElementClick(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ElementClick() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementClick; }
    static i32 static_message_id() { return (int)MessageID::ElementClick; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementClick"; }

    static ErrorOr<NonnullOwnPtr<ElementClick>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<ElementClick>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementClick));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class ElementClearResponse final : public IPC::Message {
public:

    ElementClearResponse(ElementClearResponse const&) = default;
    ElementClearResponse(ElementClearResponse&&) = default;
    ElementClearResponse& operator=(ElementClearResponse const&) = default;
    ElementClearResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ElementClearResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ElementClearResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementClearResponse; }
    static i32 static_message_id() { return (int)MessageID::ElementClearResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementClearResponse"; }

    static ErrorOr<NonnullOwnPtr<ElementClearResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ElementClearResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementClearResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ElementClear final : public IPC::Message {
public:

   typedef class ElementClearResponse ResponseType;

    ElementClear(ElementClear const&) = default;
    ElementClear(ElementClear&&) = default;
    ElementClear& operator=(ElementClear const&) = default;
    ElementClear(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    ElementClear(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ElementClear() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementClear; }
    static i32 static_message_id() { return (int)MessageID::ElementClear; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementClear"; }

    static ErrorOr<NonnullOwnPtr<ElementClear>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<ElementClear>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementClear));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class ElementSendKeysResponse final : public IPC::Message {
public:

    ElementSendKeysResponse(ElementSendKeysResponse const&) = default;
    ElementSendKeysResponse(ElementSendKeysResponse&&) = default;
    ElementSendKeysResponse& operator=(ElementSendKeysResponse const&) = default;
    ElementSendKeysResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ElementSendKeysResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ElementSendKeysResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementSendKeysResponse; }
    static i32 static_message_id() { return (int)MessageID::ElementSendKeysResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementSendKeysResponse"; }

    static ErrorOr<NonnullOwnPtr<ElementSendKeysResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ElementSendKeysResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementSendKeysResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ElementSendKeys final : public IPC::Message {
public:

   typedef class ElementSendKeysResponse ResponseType;

    ElementSendKeys(ElementSendKeys const&) = default;
    ElementSendKeys(ElementSendKeys&&) = default;
    ElementSendKeys& operator=(ElementSendKeys const&) = default;
    ElementSendKeys(String element_id, JsonValue payload) : m_element_id(move(element_id)), m_payload(move(payload)) {}

    virtual ~ElementSendKeys() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ElementSendKeys; }
    static i32 static_message_id() { return (int)MessageID::ElementSendKeys; }
    virtual const char* message_name() const override { return "WebDriverClient::ElementSendKeys"; }

    static ErrorOr<NonnullOwnPtr<ElementSendKeys>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<ElementSendKeys>(move(element_id), move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ElementSendKeys));

        TRY(stream.encode(m_element_id));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    String m_element_id;

    JsonValue m_payload;

};

class GetSourceResponse final : public IPC::Message {
public:

    GetSourceResponse(GetSourceResponse const&) = default;
    GetSourceResponse(GetSourceResponse&&) = default;
    GetSourceResponse& operator=(GetSourceResponse const&) = default;
    GetSourceResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetSourceResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetSourceResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetSourceResponse; }
    static i32 static_message_id() { return (int)MessageID::GetSourceResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetSourceResponse"; }

    static ErrorOr<NonnullOwnPtr<GetSourceResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetSourceResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetSourceResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetSource final : public IPC::Message {
public:

   typedef class GetSourceResponse ResponseType;

    GetSource(GetSource const&) = default;
    GetSource(GetSource&&) = default;
    GetSource& operator=(GetSource const&) = default;
    GetSource() {}

    virtual ~GetSource() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetSource; }
    static i32 static_message_id() { return (int)MessageID::GetSource; }
    virtual const char* message_name() const override { return "WebDriverClient::GetSource"; }

    static ErrorOr<NonnullOwnPtr<GetSource>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetSource>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetSource));

        return buffer;
    }

private:

};

class ExecuteScriptResponse final : public IPC::Message {
public:

    ExecuteScriptResponse(ExecuteScriptResponse const&) = default;
    ExecuteScriptResponse(ExecuteScriptResponse&&) = default;
    ExecuteScriptResponse& operator=(ExecuteScriptResponse const&) = default;
    ExecuteScriptResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ExecuteScriptResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ExecuteScriptResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ExecuteScriptResponse; }
    static i32 static_message_id() { return (int)MessageID::ExecuteScriptResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ExecuteScriptResponse"; }

    static ErrorOr<NonnullOwnPtr<ExecuteScriptResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ExecuteScriptResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ExecuteScriptResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ExecuteScript final : public IPC::Message {
public:

   typedef class ExecuteScriptResponse ResponseType;

    ExecuteScript(ExecuteScript const&) = default;
    ExecuteScript(ExecuteScript&&) = default;
    ExecuteScript& operator=(ExecuteScript const&) = default;
    ExecuteScript(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    ExecuteScript(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ExecuteScript() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ExecuteScript; }
    static i32 static_message_id() { return (int)MessageID::ExecuteScript; }
    virtual const char* message_name() const override { return "WebDriverClient::ExecuteScript"; }

    static ErrorOr<NonnullOwnPtr<ExecuteScript>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<ExecuteScript>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ExecuteScript));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class ExecuteAsyncScriptResponse final : public IPC::Message {
public:

    ExecuteAsyncScriptResponse(ExecuteAsyncScriptResponse const&) = default;
    ExecuteAsyncScriptResponse(ExecuteAsyncScriptResponse&&) = default;
    ExecuteAsyncScriptResponse& operator=(ExecuteAsyncScriptResponse const&) = default;
    ExecuteAsyncScriptResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ExecuteAsyncScriptResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ExecuteAsyncScriptResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ExecuteAsyncScriptResponse; }
    static i32 static_message_id() { return (int)MessageID::ExecuteAsyncScriptResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ExecuteAsyncScriptResponse"; }

    static ErrorOr<NonnullOwnPtr<ExecuteAsyncScriptResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ExecuteAsyncScriptResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ExecuteAsyncScriptResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ExecuteAsyncScript final : public IPC::Message {
public:

   typedef class ExecuteAsyncScriptResponse ResponseType;

    ExecuteAsyncScript(ExecuteAsyncScript const&) = default;
    ExecuteAsyncScript(ExecuteAsyncScript&&) = default;
    ExecuteAsyncScript& operator=(ExecuteAsyncScript const&) = default;
    ExecuteAsyncScript(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    ExecuteAsyncScript(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ExecuteAsyncScript() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ExecuteAsyncScript; }
    static i32 static_message_id() { return (int)MessageID::ExecuteAsyncScript; }
    virtual const char* message_name() const override { return "WebDriverClient::ExecuteAsyncScript"; }

    static ErrorOr<NonnullOwnPtr<ExecuteAsyncScript>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<ExecuteAsyncScript>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ExecuteAsyncScript));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class GetAllCookiesResponse final : public IPC::Message {
public:

    GetAllCookiesResponse(GetAllCookiesResponse const&) = default;
    GetAllCookiesResponse(GetAllCookiesResponse&&) = default;
    GetAllCookiesResponse& operator=(GetAllCookiesResponse const&) = default;
    GetAllCookiesResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetAllCookiesResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetAllCookiesResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetAllCookiesResponse; }
    static i32 static_message_id() { return (int)MessageID::GetAllCookiesResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetAllCookiesResponse"; }

    static ErrorOr<NonnullOwnPtr<GetAllCookiesResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetAllCookiesResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetAllCookiesResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetAllCookies final : public IPC::Message {
public:

   typedef class GetAllCookiesResponse ResponseType;

    GetAllCookies(GetAllCookies const&) = default;
    GetAllCookies(GetAllCookies&&) = default;
    GetAllCookies& operator=(GetAllCookies const&) = default;
    GetAllCookies() {}

    virtual ~GetAllCookies() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetAllCookies; }
    static i32 static_message_id() { return (int)MessageID::GetAllCookies; }
    virtual const char* message_name() const override { return "WebDriverClient::GetAllCookies"; }

    static ErrorOr<NonnullOwnPtr<GetAllCookies>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetAllCookies>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetAllCookies));

        return buffer;
    }

private:

};

class GetNamedCookieResponse final : public IPC::Message {
public:

    GetNamedCookieResponse(GetNamedCookieResponse const&) = default;
    GetNamedCookieResponse(GetNamedCookieResponse&&) = default;
    GetNamedCookieResponse& operator=(GetNamedCookieResponse const&) = default;
    GetNamedCookieResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetNamedCookieResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetNamedCookieResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetNamedCookieResponse; }
    static i32 static_message_id() { return (int)MessageID::GetNamedCookieResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetNamedCookieResponse"; }

    static ErrorOr<NonnullOwnPtr<GetNamedCookieResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetNamedCookieResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetNamedCookieResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetNamedCookie final : public IPC::Message {
public:

   typedef class GetNamedCookieResponse ResponseType;

    GetNamedCookie(GetNamedCookie const&) = default;
    GetNamedCookie(GetNamedCookie&&) = default;
    GetNamedCookie& operator=(GetNamedCookie const&) = default;
    GetNamedCookie(String name) : m_name(move(name)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    GetNamedCookie(WrappedReturnType&& value)
        : m_name(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetNamedCookie() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetNamedCookie; }
    static i32 static_message_id() { return (int)MessageID::GetNamedCookie; }
    virtual const char* message_name() const override { return "WebDriverClient::GetNamedCookie"; }

    static ErrorOr<NonnullOwnPtr<GetNamedCookie>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto name = TRY((decoder.decode<String>()));

        return make<GetNamedCookie>(move(name));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetNamedCookie));

        TRY(stream.encode(m_name));

        return buffer;
    }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

private:

    String m_name;

};

class AddCookieResponse final : public IPC::Message {
public:

    AddCookieResponse(AddCookieResponse const&) = default;
    AddCookieResponse(AddCookieResponse&&) = default;
    AddCookieResponse& operator=(AddCookieResponse const&) = default;
    AddCookieResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    AddCookieResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~AddCookieResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::AddCookieResponse; }
    static i32 static_message_id() { return (int)MessageID::AddCookieResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::AddCookieResponse"; }

    static ErrorOr<NonnullOwnPtr<AddCookieResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<AddCookieResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::AddCookieResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class AddCookie final : public IPC::Message {
public:

   typedef class AddCookieResponse ResponseType;

    AddCookie(AddCookie const&) = default;
    AddCookie(AddCookie&&) = default;
    AddCookie& operator=(AddCookie const&) = default;
    AddCookie(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    AddCookie(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~AddCookie() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::AddCookie; }
    static i32 static_message_id() { return (int)MessageID::AddCookie; }
    virtual const char* message_name() const override { return "WebDriverClient::AddCookie"; }

    static ErrorOr<NonnullOwnPtr<AddCookie>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<AddCookie>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::AddCookie));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class DeleteCookieResponse final : public IPC::Message {
public:

    DeleteCookieResponse(DeleteCookieResponse const&) = default;
    DeleteCookieResponse(DeleteCookieResponse&&) = default;
    DeleteCookieResponse& operator=(DeleteCookieResponse const&) = default;
    DeleteCookieResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    DeleteCookieResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~DeleteCookieResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DeleteCookieResponse; }
    static i32 static_message_id() { return (int)MessageID::DeleteCookieResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::DeleteCookieResponse"; }

    static ErrorOr<NonnullOwnPtr<DeleteCookieResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<DeleteCookieResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DeleteCookieResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class DeleteCookie final : public IPC::Message {
public:

   typedef class DeleteCookieResponse ResponseType;

    DeleteCookie(DeleteCookie const&) = default;
    DeleteCookie(DeleteCookie&&) = default;
    DeleteCookie& operator=(DeleteCookie const&) = default;
    DeleteCookie(String name) : m_name(move(name)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    DeleteCookie(WrappedReturnType&& value)
        : m_name(forward<WrappedReturnType>(value))
    {
    }

    virtual ~DeleteCookie() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DeleteCookie; }
    static i32 static_message_id() { return (int)MessageID::DeleteCookie; }
    virtual const char* message_name() const override { return "WebDriverClient::DeleteCookie"; }

    static ErrorOr<NonnullOwnPtr<DeleteCookie>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto name = TRY((decoder.decode<String>()));

        return make<DeleteCookie>(move(name));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DeleteCookie));

        TRY(stream.encode(m_name));

        return buffer;
    }

    const String& name() const { return m_name; }
    String take_name() { return move(m_name); }

private:

    String m_name;

};

class DeleteAllCookiesResponse final : public IPC::Message {
public:

    DeleteAllCookiesResponse(DeleteAllCookiesResponse const&) = default;
    DeleteAllCookiesResponse(DeleteAllCookiesResponse&&) = default;
    DeleteAllCookiesResponse& operator=(DeleteAllCookiesResponse const&) = default;
    DeleteAllCookiesResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    DeleteAllCookiesResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~DeleteAllCookiesResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DeleteAllCookiesResponse; }
    static i32 static_message_id() { return (int)MessageID::DeleteAllCookiesResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::DeleteAllCookiesResponse"; }

    static ErrorOr<NonnullOwnPtr<DeleteAllCookiesResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<DeleteAllCookiesResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DeleteAllCookiesResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class DeleteAllCookies final : public IPC::Message {
public:

   typedef class DeleteAllCookiesResponse ResponseType;

    DeleteAllCookies(DeleteAllCookies const&) = default;
    DeleteAllCookies(DeleteAllCookies&&) = default;
    DeleteAllCookies& operator=(DeleteAllCookies const&) = default;
    DeleteAllCookies() {}

    virtual ~DeleteAllCookies() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DeleteAllCookies; }
    static i32 static_message_id() { return (int)MessageID::DeleteAllCookies; }
    virtual const char* message_name() const override { return "WebDriverClient::DeleteAllCookies"; }

    static ErrorOr<NonnullOwnPtr<DeleteAllCookies>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<DeleteAllCookies>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DeleteAllCookies));

        return buffer;
    }

private:

};

class PerformActionsResponse final : public IPC::Message {
public:

    PerformActionsResponse(PerformActionsResponse const&) = default;
    PerformActionsResponse(PerformActionsResponse&&) = default;
    PerformActionsResponse& operator=(PerformActionsResponse const&) = default;
    PerformActionsResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    PerformActionsResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~PerformActionsResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::PerformActionsResponse; }
    static i32 static_message_id() { return (int)MessageID::PerformActionsResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::PerformActionsResponse"; }

    static ErrorOr<NonnullOwnPtr<PerformActionsResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<PerformActionsResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::PerformActionsResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class PerformActions final : public IPC::Message {
public:

   typedef class PerformActionsResponse ResponseType;

    PerformActions(PerformActions const&) = default;
    PerformActions(PerformActions&&) = default;
    PerformActions& operator=(PerformActions const&) = default;
    PerformActions(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    PerformActions(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~PerformActions() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::PerformActions; }
    static i32 static_message_id() { return (int)MessageID::PerformActions; }
    virtual const char* message_name() const override { return "WebDriverClient::PerformActions"; }

    static ErrorOr<NonnullOwnPtr<PerformActions>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<PerformActions>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::PerformActions));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class ReleaseActionsResponse final : public IPC::Message {
public:

    ReleaseActionsResponse(ReleaseActionsResponse const&) = default;
    ReleaseActionsResponse(ReleaseActionsResponse&&) = default;
    ReleaseActionsResponse& operator=(ReleaseActionsResponse const&) = default;
    ReleaseActionsResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    ReleaseActionsResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~ReleaseActionsResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ReleaseActionsResponse; }
    static i32 static_message_id() { return (int)MessageID::ReleaseActionsResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::ReleaseActionsResponse"; }

    static ErrorOr<NonnullOwnPtr<ReleaseActionsResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<ReleaseActionsResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ReleaseActionsResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class ReleaseActions final : public IPC::Message {
public:

   typedef class ReleaseActionsResponse ResponseType;

    ReleaseActions(ReleaseActions const&) = default;
    ReleaseActions(ReleaseActions&&) = default;
    ReleaseActions& operator=(ReleaseActions const&) = default;
    ReleaseActions() {}

    virtual ~ReleaseActions() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::ReleaseActions; }
    static i32 static_message_id() { return (int)MessageID::ReleaseActions; }
    virtual const char* message_name() const override { return "WebDriverClient::ReleaseActions"; }

    static ErrorOr<NonnullOwnPtr<ReleaseActions>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<ReleaseActions>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::ReleaseActions));

        return buffer;
    }

private:

};

class DismissAlertResponse final : public IPC::Message {
public:

    DismissAlertResponse(DismissAlertResponse const&) = default;
    DismissAlertResponse(DismissAlertResponse&&) = default;
    DismissAlertResponse& operator=(DismissAlertResponse const&) = default;
    DismissAlertResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    DismissAlertResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~DismissAlertResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DismissAlertResponse; }
    static i32 static_message_id() { return (int)MessageID::DismissAlertResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::DismissAlertResponse"; }

    static ErrorOr<NonnullOwnPtr<DismissAlertResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<DismissAlertResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DismissAlertResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class DismissAlert final : public IPC::Message {
public:

   typedef class DismissAlertResponse ResponseType;

    DismissAlert(DismissAlert const&) = default;
    DismissAlert(DismissAlert&&) = default;
    DismissAlert& operator=(DismissAlert const&) = default;
    DismissAlert() {}

    virtual ~DismissAlert() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::DismissAlert; }
    static i32 static_message_id() { return (int)MessageID::DismissAlert; }
    virtual const char* message_name() const override { return "WebDriverClient::DismissAlert"; }

    static ErrorOr<NonnullOwnPtr<DismissAlert>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<DismissAlert>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::DismissAlert));

        return buffer;
    }

private:

};

class AcceptAlertResponse final : public IPC::Message {
public:

    AcceptAlertResponse(AcceptAlertResponse const&) = default;
    AcceptAlertResponse(AcceptAlertResponse&&) = default;
    AcceptAlertResponse& operator=(AcceptAlertResponse const&) = default;
    AcceptAlertResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    AcceptAlertResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~AcceptAlertResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::AcceptAlertResponse; }
    static i32 static_message_id() { return (int)MessageID::AcceptAlertResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::AcceptAlertResponse"; }

    static ErrorOr<NonnullOwnPtr<AcceptAlertResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<AcceptAlertResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::AcceptAlertResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class AcceptAlert final : public IPC::Message {
public:

   typedef class AcceptAlertResponse ResponseType;

    AcceptAlert(AcceptAlert const&) = default;
    AcceptAlert(AcceptAlert&&) = default;
    AcceptAlert& operator=(AcceptAlert const&) = default;
    AcceptAlert() {}

    virtual ~AcceptAlert() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::AcceptAlert; }
    static i32 static_message_id() { return (int)MessageID::AcceptAlert; }
    virtual const char* message_name() const override { return "WebDriverClient::AcceptAlert"; }

    static ErrorOr<NonnullOwnPtr<AcceptAlert>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<AcceptAlert>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::AcceptAlert));

        return buffer;
    }

private:

};

class GetAlertTextResponse final : public IPC::Message {
public:

    GetAlertTextResponse(GetAlertTextResponse const&) = default;
    GetAlertTextResponse(GetAlertTextResponse&&) = default;
    GetAlertTextResponse& operator=(GetAlertTextResponse const&) = default;
    GetAlertTextResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    GetAlertTextResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~GetAlertTextResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetAlertTextResponse; }
    static i32 static_message_id() { return (int)MessageID::GetAlertTextResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::GetAlertTextResponse"; }

    static ErrorOr<NonnullOwnPtr<GetAlertTextResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<GetAlertTextResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetAlertTextResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class GetAlertText final : public IPC::Message {
public:

   typedef class GetAlertTextResponse ResponseType;

    GetAlertText(GetAlertText const&) = default;
    GetAlertText(GetAlertText&&) = default;
    GetAlertText& operator=(GetAlertText const&) = default;
    GetAlertText() {}

    virtual ~GetAlertText() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::GetAlertText; }
    static i32 static_message_id() { return (int)MessageID::GetAlertText; }
    virtual const char* message_name() const override { return "WebDriverClient::GetAlertText"; }

    static ErrorOr<NonnullOwnPtr<GetAlertText>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<GetAlertText>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::GetAlertText));

        return buffer;
    }

private:

};

class SendAlertTextResponse final : public IPC::Message {
public:

    SendAlertTextResponse(SendAlertTextResponse const&) = default;
    SendAlertTextResponse(SendAlertTextResponse&&) = default;
    SendAlertTextResponse& operator=(SendAlertTextResponse const&) = default;
    SendAlertTextResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    SendAlertTextResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SendAlertTextResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SendAlertTextResponse; }
    static i32 static_message_id() { return (int)MessageID::SendAlertTextResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::SendAlertTextResponse"; }

    static ErrorOr<NonnullOwnPtr<SendAlertTextResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<SendAlertTextResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SendAlertTextResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class SendAlertText final : public IPC::Message {
public:

   typedef class SendAlertTextResponse ResponseType;

    SendAlertText(SendAlertText const&) = default;
    SendAlertText(SendAlertText&&) = default;
    SendAlertText& operator=(SendAlertText const&) = default;
    SendAlertText(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    SendAlertText(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~SendAlertText() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::SendAlertText; }
    static i32 static_message_id() { return (int)MessageID::SendAlertText; }
    virtual const char* message_name() const override { return "WebDriverClient::SendAlertText"; }

    static ErrorOr<NonnullOwnPtr<SendAlertText>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<SendAlertText>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::SendAlertText));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class TakeScreenshotResponse final : public IPC::Message {
public:

    TakeScreenshotResponse(TakeScreenshotResponse const&) = default;
    TakeScreenshotResponse(TakeScreenshotResponse&&) = default;
    TakeScreenshotResponse& operator=(TakeScreenshotResponse const&) = default;
    TakeScreenshotResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    TakeScreenshotResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~TakeScreenshotResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::TakeScreenshotResponse; }
    static i32 static_message_id() { return (int)MessageID::TakeScreenshotResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::TakeScreenshotResponse"; }

    static ErrorOr<NonnullOwnPtr<TakeScreenshotResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<TakeScreenshotResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::TakeScreenshotResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class TakeScreenshot final : public IPC::Message {
public:

   typedef class TakeScreenshotResponse ResponseType;

    TakeScreenshot(TakeScreenshot const&) = default;
    TakeScreenshot(TakeScreenshot&&) = default;
    TakeScreenshot& operator=(TakeScreenshot const&) = default;
    TakeScreenshot() {}

    virtual ~TakeScreenshot() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::TakeScreenshot; }
    static i32 static_message_id() { return (int)MessageID::TakeScreenshot; }
    virtual const char* message_name() const override { return "WebDriverClient::TakeScreenshot"; }

    static ErrorOr<NonnullOwnPtr<TakeScreenshot>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<TakeScreenshot>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::TakeScreenshot));

        return buffer;
    }

private:

};

class TakeElementScreenshotResponse final : public IPC::Message {
public:

    TakeElementScreenshotResponse(TakeElementScreenshotResponse const&) = default;
    TakeElementScreenshotResponse(TakeElementScreenshotResponse&&) = default;
    TakeElementScreenshotResponse& operator=(TakeElementScreenshotResponse const&) = default;
    TakeElementScreenshotResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    TakeElementScreenshotResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~TakeElementScreenshotResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::TakeElementScreenshotResponse; }
    static i32 static_message_id() { return (int)MessageID::TakeElementScreenshotResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::TakeElementScreenshotResponse"; }

    static ErrorOr<NonnullOwnPtr<TakeElementScreenshotResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<TakeElementScreenshotResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::TakeElementScreenshotResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class TakeElementScreenshot final : public IPC::Message {
public:

   typedef class TakeElementScreenshotResponse ResponseType;

    TakeElementScreenshot(TakeElementScreenshot const&) = default;
    TakeElementScreenshot(TakeElementScreenshot&&) = default;
    TakeElementScreenshot& operator=(TakeElementScreenshot const&) = default;
    TakeElementScreenshot(String element_id) : m_element_id(move(element_id)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, String>)
    TakeElementScreenshot(WrappedReturnType&& value)
        : m_element_id(forward<WrappedReturnType>(value))
    {
    }

    virtual ~TakeElementScreenshot() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::TakeElementScreenshot; }
    static i32 static_message_id() { return (int)MessageID::TakeElementScreenshot; }
    virtual const char* message_name() const override { return "WebDriverClient::TakeElementScreenshot"; }

    static ErrorOr<NonnullOwnPtr<TakeElementScreenshot>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto element_id = TRY((decoder.decode<String>()));

        return make<TakeElementScreenshot>(move(element_id));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::TakeElementScreenshot));

        TRY(stream.encode(m_element_id));

        return buffer;
    }

    const String& element_id() const { return m_element_id; }
    String take_element_id() { return move(m_element_id); }

private:

    String m_element_id;

};

class PrintPageResponse final : public IPC::Message {
public:

    PrintPageResponse(PrintPageResponse const&) = default;
    PrintPageResponse(PrintPageResponse&&) = default;
    PrintPageResponse& operator=(PrintPageResponse const&) = default;
    PrintPageResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    PrintPageResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~PrintPageResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::PrintPageResponse; }
    static i32 static_message_id() { return (int)MessageID::PrintPageResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::PrintPageResponse"; }

    static ErrorOr<NonnullOwnPtr<PrintPageResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<PrintPageResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::PrintPageResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class PrintPage final : public IPC::Message {
public:

   typedef class PrintPageResponse ResponseType;

    PrintPage(PrintPage const&) = default;
    PrintPage(PrintPage&&) = default;
    PrintPage& operator=(PrintPage const&) = default;
    PrintPage(JsonValue payload) : m_payload(move(payload)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, JsonValue>)
    PrintPage(WrappedReturnType&& value)
        : m_payload(forward<WrappedReturnType>(value))
    {
    }

    virtual ~PrintPage() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::PrintPage; }
    static i32 static_message_id() { return (int)MessageID::PrintPage; }
    virtual const char* message_name() const override { return "WebDriverClient::PrintPage"; }

    static ErrorOr<NonnullOwnPtr<PrintPage>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto payload = TRY((decoder.decode<JsonValue>()));

        return make<PrintPage>(move(payload));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::PrintPage));

        TRY(stream.encode(m_payload));

        return buffer;
    }

    const JsonValue& payload() const { return m_payload; }
    JsonValue take_payload() { return move(m_payload); }

private:

    JsonValue m_payload;

};

class EnsureTopLevelBrowsingContextIsOpenResponse final : public IPC::Message {
public:

    EnsureTopLevelBrowsingContextIsOpenResponse(EnsureTopLevelBrowsingContextIsOpenResponse const&) = default;
    EnsureTopLevelBrowsingContextIsOpenResponse(EnsureTopLevelBrowsingContextIsOpenResponse&&) = default;
    EnsureTopLevelBrowsingContextIsOpenResponse& operator=(EnsureTopLevelBrowsingContextIsOpenResponse const&) = default;
    EnsureTopLevelBrowsingContextIsOpenResponse(Web::WebDriver::Response response) : m_response(move(response)) {}

    template <typename WrappedReturnType>
    requires(!SameAs<WrappedReturnType, Web::WebDriver::Response>)
    EnsureTopLevelBrowsingContextIsOpenResponse(WrappedReturnType&& value)
        : m_response(forward<WrappedReturnType>(value))
    {
    }

    virtual ~EnsureTopLevelBrowsingContextIsOpenResponse() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::EnsureTopLevelBrowsingContextIsOpenResponse; }
    static i32 static_message_id() { return (int)MessageID::EnsureTopLevelBrowsingContextIsOpenResponse; }
    virtual const char* message_name() const override { return "WebDriverClient::EnsureTopLevelBrowsingContextIsOpenResponse"; }

    static ErrorOr<NonnullOwnPtr<EnsureTopLevelBrowsingContextIsOpenResponse>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        auto response = TRY((decoder.decode<Web::WebDriver::Response>()));

        return make<EnsureTopLevelBrowsingContextIsOpenResponse>(move(response));
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::EnsureTopLevelBrowsingContextIsOpenResponse));

        TRY(stream.encode(m_response));

        return buffer;
    }

    const Web::WebDriver::Response& response() const { return m_response; }
    Web::WebDriver::Response take_response() { return move(m_response); }

private:

    Web::WebDriver::Response m_response;

};

class EnsureTopLevelBrowsingContextIsOpen final : public IPC::Message {
public:

   typedef class EnsureTopLevelBrowsingContextIsOpenResponse ResponseType;

    EnsureTopLevelBrowsingContextIsOpen(EnsureTopLevelBrowsingContextIsOpen const&) = default;
    EnsureTopLevelBrowsingContextIsOpen(EnsureTopLevelBrowsingContextIsOpen&&) = default;
    EnsureTopLevelBrowsingContextIsOpen& operator=(EnsureTopLevelBrowsingContextIsOpen const&) = default;
    EnsureTopLevelBrowsingContextIsOpen() {}

    virtual ~EnsureTopLevelBrowsingContextIsOpen() override {}

    virtual u32 endpoint_magic() const override { return 4284153835; }
    virtual i32 message_id() const override { return (int)MessageID::EnsureTopLevelBrowsingContextIsOpen; }
    static i32 static_message_id() { return (int)MessageID::EnsureTopLevelBrowsingContextIsOpen; }
    virtual const char* message_name() const override { return "WebDriverClient::EnsureTopLevelBrowsingContextIsOpen"; }

    static ErrorOr<NonnullOwnPtr<EnsureTopLevelBrowsingContextIsOpen>> decode(Stream& stream, Queue<IPC::File>& files)
    {
        IPC::Decoder decoder { stream, files };

        return make<EnsureTopLevelBrowsingContextIsOpen>();
    }

    virtual ErrorOr<IPC::MessageBuffer> encode() const override
    {
        IPC::MessageBuffer buffer;
        IPC::Encoder stream(buffer);
        TRY(stream.encode(endpoint_magic()));
        TRY(stream.encode((int)MessageID::EnsureTopLevelBrowsingContextIsOpen));

        return buffer;
    }

private:

};

} // namespace Messages::WebDriverClient

template<typename LocalEndpoint, typename PeerEndpoint>
class WebDriverClientProxy {
public:
    // Used to disambiguate the constructor call.
    struct Tag { };

    WebDriverClientProxy(IPC::Connection<LocalEndpoint, PeerEndpoint>& connection, Tag)
        : m_connection(connection)
    { }

    void close_session(
) {
        (void) m_connection.template send_sync<Messages::WebDriverClient::CloseSession>();
    }

    void async_close_session(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::CloseSession {  }));

    }

    IPC::IPCErrorOr<void> try_close_session(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::CloseSession>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return { };

    }

    void async_set_page_load_strategy(
Web::WebDriver::PageLoadStrategy page_load_strategy) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetPageLoadStrategy { move(page_load_strategy) }));

    }

    void async_set_user_prompt_handler(
Web::WebDriver::UserPromptHandler user_prompt_handler) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetUserPromptHandler { move(user_prompt_handler) }));

    }

    void async_set_strict_file_interactability(
bool strict_file_interactability) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetStrictFileInteractability { strict_file_interactability }));

    }

    void async_set_is_webdriver_active(
bool active) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetIsWebdriverActive { active }));

    }

    Web::WebDriver::Response get_timeouts(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetTimeouts>()->take_response();
    }

    void async_get_timeouts(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetTimeouts {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_timeouts(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetTimeouts>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response set_timeouts(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::SetTimeouts>(move(payload))->take_response();
    }

    void async_set_timeouts(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetTimeouts { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_set_timeouts(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SetTimeouts>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response navigate_to(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::NavigateTo>(move(payload))->take_response();
    }

    void async_navigate_to(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::NavigateTo { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_navigate_to(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::NavigateTo>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_current_url(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetCurrentUrl>()->take_response();
    }

    void async_get_current_url(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetCurrentUrl {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_current_url(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetCurrentUrl>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response back(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::Back>()->take_response();
    }

    void async_back(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::Back {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_back(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::Back>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response forward(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::Forward>()->take_response();
    }

    void async_forward(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::Forward {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_forward(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::Forward>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response refresh(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::Refresh>()->take_response();
    }

    void async_refresh(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::Refresh {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_refresh(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::Refresh>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_title(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetTitle>()->take_response();
    }

    void async_get_title(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetTitle {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_title(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetTitle>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_window_handle(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetWindowHandle>()->take_response();
    }

    void async_get_window_handle(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetWindowHandle {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_window_handle(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetWindowHandle>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response close_window(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::CloseWindow>()->take_response();
    }

    void async_close_window(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::CloseWindow {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_close_window(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::CloseWindow>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response switch_to_window(
String handle) {
        return m_connection.template send_sync<Messages::WebDriverClient::SwitchToWindow>(move(handle))->take_response();
    }

    void async_switch_to_window(
String handle) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SwitchToWindow { move(handle) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_switch_to_window(
String handle) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SwitchToWindow>(move(handle));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response new_window(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::NewWindow>(move(payload))->take_response();
    }

    void async_new_window(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::NewWindow { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_new_window(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::NewWindow>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response switch_to_frame(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::SwitchToFrame>(move(payload))->take_response();
    }

    void async_switch_to_frame(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SwitchToFrame { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_switch_to_frame(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SwitchToFrame>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response switch_to_parent_frame(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::SwitchToParentFrame>(move(payload))->take_response();
    }

    void async_switch_to_parent_frame(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SwitchToParentFrame { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_switch_to_parent_frame(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SwitchToParentFrame>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_window_rect(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetWindowRect>()->take_response();
    }

    void async_get_window_rect(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetWindowRect {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_window_rect(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetWindowRect>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response set_window_rect(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::SetWindowRect>(move(payload))->take_response();
    }

    void async_set_window_rect(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SetWindowRect { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_set_window_rect(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SetWindowRect>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response maximize_window(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::MaximizeWindow>()->take_response();
    }

    void async_maximize_window(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::MaximizeWindow {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_maximize_window(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::MaximizeWindow>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response minimize_window(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::MinimizeWindow>()->take_response();
    }

    void async_minimize_window(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::MinimizeWindow {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_minimize_window(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::MinimizeWindow>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response fullscreen_window(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::FullscreenWindow>()->take_response();
    }

    void async_fullscreen_window(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FullscreenWindow {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_fullscreen_window(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FullscreenWindow>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response consume_user_activation(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::ConsumeUserActivation>()->take_response();
    }

    void async_consume_user_activation(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ConsumeUserActivation {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_consume_user_activation(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ConsumeUserActivation>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_element(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElement>(move(payload))->take_response();
    }

    void async_find_element(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElement { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_element(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElement>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_elements(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElements>(move(payload))->take_response();
    }

    void async_find_elements(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElements { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_elements(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElements>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_element_from_element(
JsonValue payload, String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElementFromElement>(move(payload), move(element_id))->take_response();
    }

    void async_find_element_from_element(
JsonValue payload, String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElementFromElement { move(payload), move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_element_from_element(
JsonValue payload, String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElementFromElement>(move(payload), move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_elements_from_element(
JsonValue payload, String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElementsFromElement>(move(payload), move(element_id))->take_response();
    }

    void async_find_elements_from_element(
JsonValue payload, String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElementsFromElement { move(payload), move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_elements_from_element(
JsonValue payload, String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElementsFromElement>(move(payload), move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_element_from_shadow_root(
JsonValue payload, String shadow_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElementFromShadowRoot>(move(payload), move(shadow_id))->take_response();
    }

    void async_find_element_from_shadow_root(
JsonValue payload, String shadow_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElementFromShadowRoot { move(payload), move(shadow_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_element_from_shadow_root(
JsonValue payload, String shadow_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElementFromShadowRoot>(move(payload), move(shadow_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response find_elements_from_shadow_root(
JsonValue payload, String shadow_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::FindElementsFromShadowRoot>(move(payload), move(shadow_id))->take_response();
    }

    void async_find_elements_from_shadow_root(
JsonValue payload, String shadow_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::FindElementsFromShadowRoot { move(payload), move(shadow_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_find_elements_from_shadow_root(
JsonValue payload, String shadow_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::FindElementsFromShadowRoot>(move(payload), move(shadow_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_active_element(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetActiveElement>()->take_response();
    }

    void async_get_active_element(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetActiveElement {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_active_element(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetActiveElement>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_shadow_root(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementShadowRoot>(move(element_id))->take_response();
    }

    void async_get_element_shadow_root(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementShadowRoot { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_shadow_root(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementShadowRoot>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response is_element_selected(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::IsElementSelected>(move(element_id))->take_response();
    }

    void async_is_element_selected(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::IsElementSelected { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_is_element_selected(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::IsElementSelected>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_attribute(
String element_id, String name) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementAttribute>(move(element_id), move(name))->take_response();
    }

    void async_get_element_attribute(
String element_id, String name) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementAttribute { move(element_id), move(name) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_attribute(
String element_id, String name) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementAttribute>(move(element_id), move(name));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_property(
String element_id, String name) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementProperty>(move(element_id), move(name))->take_response();
    }

    void async_get_element_property(
String element_id, String name) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementProperty { move(element_id), move(name) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_property(
String element_id, String name) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementProperty>(move(element_id), move(name));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_css_value(
String element_id, String name) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementCssValue>(move(element_id), move(name))->take_response();
    }

    void async_get_element_css_value(
String element_id, String name) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementCssValue { move(element_id), move(name) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_css_value(
String element_id, String name) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementCssValue>(move(element_id), move(name));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_text(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementText>(move(element_id))->take_response();
    }

    void async_get_element_text(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementText { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_text(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementText>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_tag_name(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementTagName>(move(element_id))->take_response();
    }

    void async_get_element_tag_name(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementTagName { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_tag_name(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementTagName>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_element_rect(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetElementRect>(move(element_id))->take_response();
    }

    void async_get_element_rect(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetElementRect { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_element_rect(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetElementRect>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response is_element_enabled(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::IsElementEnabled>(move(element_id))->take_response();
    }

    void async_is_element_enabled(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::IsElementEnabled { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_is_element_enabled(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::IsElementEnabled>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_computed_role(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetComputedRole>(move(element_id))->take_response();
    }

    void async_get_computed_role(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetComputedRole { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_computed_role(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetComputedRole>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_computed_label(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetComputedLabel>(move(element_id))->take_response();
    }

    void async_get_computed_label(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetComputedLabel { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_computed_label(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetComputedLabel>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response element_click(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::ElementClick>(move(element_id))->take_response();
    }

    void async_element_click(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ElementClick { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_element_click(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ElementClick>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response element_clear(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::ElementClear>(move(element_id))->take_response();
    }

    void async_element_clear(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ElementClear { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_element_clear(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ElementClear>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response element_send_keys(
String element_id, JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::ElementSendKeys>(move(element_id), move(payload))->take_response();
    }

    void async_element_send_keys(
String element_id, JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ElementSendKeys { move(element_id), move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_element_send_keys(
String element_id, JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ElementSendKeys>(move(element_id), move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_source(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetSource>()->take_response();
    }

    void async_get_source(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetSource {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_source(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetSource>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response execute_script(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::ExecuteScript>(move(payload))->take_response();
    }

    void async_execute_script(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ExecuteScript { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_execute_script(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ExecuteScript>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response execute_async_script(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::ExecuteAsyncScript>(move(payload))->take_response();
    }

    void async_execute_async_script(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ExecuteAsyncScript { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_execute_async_script(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ExecuteAsyncScript>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_all_cookies(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetAllCookies>()->take_response();
    }

    void async_get_all_cookies(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetAllCookies {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_all_cookies(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetAllCookies>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_named_cookie(
String name) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetNamedCookie>(move(name))->take_response();
    }

    void async_get_named_cookie(
String name) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetNamedCookie { move(name) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_named_cookie(
String name) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetNamedCookie>(move(name));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response add_cookie(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::AddCookie>(move(payload))->take_response();
    }

    void async_add_cookie(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::AddCookie { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_add_cookie(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::AddCookie>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response delete_cookie(
String name) {
        return m_connection.template send_sync<Messages::WebDriverClient::DeleteCookie>(move(name))->take_response();
    }

    void async_delete_cookie(
String name) {
        MUST(m_connection.post_message(Messages::WebDriverClient::DeleteCookie { move(name) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_delete_cookie(
String name) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::DeleteCookie>(move(name));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response delete_all_cookies(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::DeleteAllCookies>()->take_response();
    }

    void async_delete_all_cookies(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::DeleteAllCookies {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_delete_all_cookies(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::DeleteAllCookies>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response perform_actions(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::PerformActions>(move(payload))->take_response();
    }

    void async_perform_actions(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::PerformActions { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_perform_actions(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::PerformActions>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response release_actions(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::ReleaseActions>()->take_response();
    }

    void async_release_actions(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::ReleaseActions {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_release_actions(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::ReleaseActions>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response dismiss_alert(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::DismissAlert>()->take_response();
    }

    void async_dismiss_alert(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::DismissAlert {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_dismiss_alert(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::DismissAlert>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response accept_alert(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::AcceptAlert>()->take_response();
    }

    void async_accept_alert(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::AcceptAlert {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_accept_alert(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::AcceptAlert>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response get_alert_text(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::GetAlertText>()->take_response();
    }

    void async_get_alert_text(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::GetAlertText {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_get_alert_text(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::GetAlertText>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response send_alert_text(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::SendAlertText>(move(payload))->take_response();
    }

    void async_send_alert_text(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::SendAlertText { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_send_alert_text(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::SendAlertText>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response take_screenshot(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::TakeScreenshot>()->take_response();
    }

    void async_take_screenshot(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::TakeScreenshot {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_take_screenshot(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::TakeScreenshot>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response take_element_screenshot(
String element_id) {
        return m_connection.template send_sync<Messages::WebDriverClient::TakeElementScreenshot>(move(element_id))->take_response();
    }

    void async_take_element_screenshot(
String element_id) {
        MUST(m_connection.post_message(Messages::WebDriverClient::TakeElementScreenshot { move(element_id) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_take_element_screenshot(
String element_id) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::TakeElementScreenshot>(move(element_id));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response print_page(
JsonValue payload) {
        return m_connection.template send_sync<Messages::WebDriverClient::PrintPage>(move(payload))->take_response();
    }

    void async_print_page(
JsonValue payload) {
        MUST(m_connection.post_message(Messages::WebDriverClient::PrintPage { move(payload) }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_print_page(
JsonValue payload) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::PrintPage>(move(payload));
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

    Web::WebDriver::Response ensure_top_level_browsing_context_is_open(
) {
        return m_connection.template send_sync<Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpen>()->take_response();
    }

    void async_ensure_top_level_browsing_context_is_open(
) {
        MUST(m_connection.post_message(Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpen {  }));

    }

    IPC::IPCErrorOr<Web::WebDriver::Response> try_ensure_top_level_browsing_context_is_open(
) {
        auto result = m_connection.template send_sync_but_allow_failure<Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpen>();
        if (!result) {
            m_connection.shutdown();
            return IPC::ErrorCode::PeerDisconnected;
        }
        return move(*result);

    }

private:
    IPC::Connection<LocalEndpoint, PeerEndpoint>& m_connection;
};

template<typename LocalEndpoint, typename PeerEndpoint>
class WebDriverClientProxy;
class WebDriverClientStub;

class WebDriverClientEndpoint {
public:
    template<typename LocalEndpoint>
    using Proxy = WebDriverClientProxy<LocalEndpoint, WebDriverClientEndpoint>;
    using Stub = WebDriverClientStub;

    static u32 static_magic() { return 4284153835; }

    static ErrorOr<NonnullOwnPtr<IPC::Message>> decode_message(ReadonlyBytes buffer, [[maybe_unused]] Queue<IPC::File>& files)
    {
        FixedMemoryStream stream { buffer };
        auto message_endpoint_magic = TRY(stream.read_value<u32>());

        if (message_endpoint_magic != 4284153835) {
            return Error::from_string_literal("Endpoint magic number mismatch, not my message!");
        }

        auto message_id = TRY(stream.read_value<i32>());


        switch (message_id) {

        case (int)Messages::WebDriverClient::MessageID::CloseSession:
            return TRY(Messages::WebDriverClient::CloseSession::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::CloseSessionResponse:
            return TRY(Messages::WebDriverClient::CloseSessionResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetPageLoadStrategy:
            return TRY(Messages::WebDriverClient::SetPageLoadStrategy::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetUserPromptHandler:
            return TRY(Messages::WebDriverClient::SetUserPromptHandler::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetStrictFileInteractability:
            return TRY(Messages::WebDriverClient::SetStrictFileInteractability::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetIsWebdriverActive:
            return TRY(Messages::WebDriverClient::SetIsWebdriverActive::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetTimeouts:
            return TRY(Messages::WebDriverClient::GetTimeouts::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetTimeoutsResponse:
            return TRY(Messages::WebDriverClient::GetTimeoutsResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetTimeouts:
            return TRY(Messages::WebDriverClient::SetTimeouts::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetTimeoutsResponse:
            return TRY(Messages::WebDriverClient::SetTimeoutsResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::NavigateTo:
            return TRY(Messages::WebDriverClient::NavigateTo::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::NavigateToResponse:
            return TRY(Messages::WebDriverClient::NavigateToResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetCurrentUrl:
            return TRY(Messages::WebDriverClient::GetCurrentUrl::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetCurrentUrlResponse:
            return TRY(Messages::WebDriverClient::GetCurrentUrlResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::Back:
            return TRY(Messages::WebDriverClient::Back::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::BackResponse:
            return TRY(Messages::WebDriverClient::BackResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::Forward:
            return TRY(Messages::WebDriverClient::Forward::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ForwardResponse:
            return TRY(Messages::WebDriverClient::ForwardResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::Refresh:
            return TRY(Messages::WebDriverClient::Refresh::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::RefreshResponse:
            return TRY(Messages::WebDriverClient::RefreshResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetTitle:
            return TRY(Messages::WebDriverClient::GetTitle::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetTitleResponse:
            return TRY(Messages::WebDriverClient::GetTitleResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetWindowHandle:
            return TRY(Messages::WebDriverClient::GetWindowHandle::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetWindowHandleResponse:
            return TRY(Messages::WebDriverClient::GetWindowHandleResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::CloseWindow:
            return TRY(Messages::WebDriverClient::CloseWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::CloseWindowResponse:
            return TRY(Messages::WebDriverClient::CloseWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToWindow:
            return TRY(Messages::WebDriverClient::SwitchToWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToWindowResponse:
            return TRY(Messages::WebDriverClient::SwitchToWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::NewWindow:
            return TRY(Messages::WebDriverClient::NewWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::NewWindowResponse:
            return TRY(Messages::WebDriverClient::NewWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToFrame:
            return TRY(Messages::WebDriverClient::SwitchToFrame::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToFrameResponse:
            return TRY(Messages::WebDriverClient::SwitchToFrameResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToParentFrame:
            return TRY(Messages::WebDriverClient::SwitchToParentFrame::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SwitchToParentFrameResponse:
            return TRY(Messages::WebDriverClient::SwitchToParentFrameResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetWindowRect:
            return TRY(Messages::WebDriverClient::GetWindowRect::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetWindowRectResponse:
            return TRY(Messages::WebDriverClient::GetWindowRectResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetWindowRect:
            return TRY(Messages::WebDriverClient::SetWindowRect::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SetWindowRectResponse:
            return TRY(Messages::WebDriverClient::SetWindowRectResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::MaximizeWindow:
            return TRY(Messages::WebDriverClient::MaximizeWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::MaximizeWindowResponse:
            return TRY(Messages::WebDriverClient::MaximizeWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::MinimizeWindow:
            return TRY(Messages::WebDriverClient::MinimizeWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::MinimizeWindowResponse:
            return TRY(Messages::WebDriverClient::MinimizeWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FullscreenWindow:
            return TRY(Messages::WebDriverClient::FullscreenWindow::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FullscreenWindowResponse:
            return TRY(Messages::WebDriverClient::FullscreenWindowResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ConsumeUserActivation:
            return TRY(Messages::WebDriverClient::ConsumeUserActivation::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ConsumeUserActivationResponse:
            return TRY(Messages::WebDriverClient::ConsumeUserActivationResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElement:
            return TRY(Messages::WebDriverClient::FindElement::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementResponse:
            return TRY(Messages::WebDriverClient::FindElementResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElements:
            return TRY(Messages::WebDriverClient::FindElements::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementsResponse:
            return TRY(Messages::WebDriverClient::FindElementsResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementFromElement:
            return TRY(Messages::WebDriverClient::FindElementFromElement::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementFromElementResponse:
            return TRY(Messages::WebDriverClient::FindElementFromElementResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementsFromElement:
            return TRY(Messages::WebDriverClient::FindElementsFromElement::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementsFromElementResponse:
            return TRY(Messages::WebDriverClient::FindElementsFromElementResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementFromShadowRoot:
            return TRY(Messages::WebDriverClient::FindElementFromShadowRoot::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementFromShadowRootResponse:
            return TRY(Messages::WebDriverClient::FindElementFromShadowRootResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementsFromShadowRoot:
            return TRY(Messages::WebDriverClient::FindElementsFromShadowRoot::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::FindElementsFromShadowRootResponse:
            return TRY(Messages::WebDriverClient::FindElementsFromShadowRootResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetActiveElement:
            return TRY(Messages::WebDriverClient::GetActiveElement::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetActiveElementResponse:
            return TRY(Messages::WebDriverClient::GetActiveElementResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementShadowRoot:
            return TRY(Messages::WebDriverClient::GetElementShadowRoot::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementShadowRootResponse:
            return TRY(Messages::WebDriverClient::GetElementShadowRootResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::IsElementSelected:
            return TRY(Messages::WebDriverClient::IsElementSelected::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::IsElementSelectedResponse:
            return TRY(Messages::WebDriverClient::IsElementSelectedResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementAttribute:
            return TRY(Messages::WebDriverClient::GetElementAttribute::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementAttributeResponse:
            return TRY(Messages::WebDriverClient::GetElementAttributeResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementProperty:
            return TRY(Messages::WebDriverClient::GetElementProperty::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementPropertyResponse:
            return TRY(Messages::WebDriverClient::GetElementPropertyResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementCssValue:
            return TRY(Messages::WebDriverClient::GetElementCssValue::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementCssValueResponse:
            return TRY(Messages::WebDriverClient::GetElementCssValueResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementText:
            return TRY(Messages::WebDriverClient::GetElementText::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementTextResponse:
            return TRY(Messages::WebDriverClient::GetElementTextResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementTagName:
            return TRY(Messages::WebDriverClient::GetElementTagName::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementTagNameResponse:
            return TRY(Messages::WebDriverClient::GetElementTagNameResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementRect:
            return TRY(Messages::WebDriverClient::GetElementRect::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetElementRectResponse:
            return TRY(Messages::WebDriverClient::GetElementRectResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::IsElementEnabled:
            return TRY(Messages::WebDriverClient::IsElementEnabled::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::IsElementEnabledResponse:
            return TRY(Messages::WebDriverClient::IsElementEnabledResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetComputedRole:
            return TRY(Messages::WebDriverClient::GetComputedRole::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetComputedRoleResponse:
            return TRY(Messages::WebDriverClient::GetComputedRoleResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetComputedLabel:
            return TRY(Messages::WebDriverClient::GetComputedLabel::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetComputedLabelResponse:
            return TRY(Messages::WebDriverClient::GetComputedLabelResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementClick:
            return TRY(Messages::WebDriverClient::ElementClick::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementClickResponse:
            return TRY(Messages::WebDriverClient::ElementClickResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementClear:
            return TRY(Messages::WebDriverClient::ElementClear::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementClearResponse:
            return TRY(Messages::WebDriverClient::ElementClearResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementSendKeys:
            return TRY(Messages::WebDriverClient::ElementSendKeys::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ElementSendKeysResponse:
            return TRY(Messages::WebDriverClient::ElementSendKeysResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetSource:
            return TRY(Messages::WebDriverClient::GetSource::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetSourceResponse:
            return TRY(Messages::WebDriverClient::GetSourceResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ExecuteScript:
            return TRY(Messages::WebDriverClient::ExecuteScript::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ExecuteScriptResponse:
            return TRY(Messages::WebDriverClient::ExecuteScriptResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ExecuteAsyncScript:
            return TRY(Messages::WebDriverClient::ExecuteAsyncScript::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ExecuteAsyncScriptResponse:
            return TRY(Messages::WebDriverClient::ExecuteAsyncScriptResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetAllCookies:
            return TRY(Messages::WebDriverClient::GetAllCookies::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetAllCookiesResponse:
            return TRY(Messages::WebDriverClient::GetAllCookiesResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetNamedCookie:
            return TRY(Messages::WebDriverClient::GetNamedCookie::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetNamedCookieResponse:
            return TRY(Messages::WebDriverClient::GetNamedCookieResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::AddCookie:
            return TRY(Messages::WebDriverClient::AddCookie::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::AddCookieResponse:
            return TRY(Messages::WebDriverClient::AddCookieResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DeleteCookie:
            return TRY(Messages::WebDriverClient::DeleteCookie::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DeleteCookieResponse:
            return TRY(Messages::WebDriverClient::DeleteCookieResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DeleteAllCookies:
            return TRY(Messages::WebDriverClient::DeleteAllCookies::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DeleteAllCookiesResponse:
            return TRY(Messages::WebDriverClient::DeleteAllCookiesResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::PerformActions:
            return TRY(Messages::WebDriverClient::PerformActions::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::PerformActionsResponse:
            return TRY(Messages::WebDriverClient::PerformActionsResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ReleaseActions:
            return TRY(Messages::WebDriverClient::ReleaseActions::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::ReleaseActionsResponse:
            return TRY(Messages::WebDriverClient::ReleaseActionsResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DismissAlert:
            return TRY(Messages::WebDriverClient::DismissAlert::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::DismissAlertResponse:
            return TRY(Messages::WebDriverClient::DismissAlertResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::AcceptAlert:
            return TRY(Messages::WebDriverClient::AcceptAlert::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::AcceptAlertResponse:
            return TRY(Messages::WebDriverClient::AcceptAlertResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetAlertText:
            return TRY(Messages::WebDriverClient::GetAlertText::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::GetAlertTextResponse:
            return TRY(Messages::WebDriverClient::GetAlertTextResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SendAlertText:
            return TRY(Messages::WebDriverClient::SendAlertText::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::SendAlertTextResponse:
            return TRY(Messages::WebDriverClient::SendAlertTextResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::TakeScreenshot:
            return TRY(Messages::WebDriverClient::TakeScreenshot::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::TakeScreenshotResponse:
            return TRY(Messages::WebDriverClient::TakeScreenshotResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::TakeElementScreenshot:
            return TRY(Messages::WebDriverClient::TakeElementScreenshot::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::TakeElementScreenshotResponse:
            return TRY(Messages::WebDriverClient::TakeElementScreenshotResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::PrintPage:
            return TRY(Messages::WebDriverClient::PrintPage::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::PrintPageResponse:
            return TRY(Messages::WebDriverClient::PrintPageResponse::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::EnsureTopLevelBrowsingContextIsOpen:
            return TRY(Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpen::decode(stream, files));
        case (int)Messages::WebDriverClient::MessageID::EnsureTopLevelBrowsingContextIsOpenResponse:
            return TRY(Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpenResponse::decode(stream, files));
        default:
            return Error::from_string_literal("Failed to decode WebDriverClient message");
        }

        VERIFY_NOT_REACHED();
    }

};

class WebDriverClientStub : public IPC::Stub {
public:
    WebDriverClientStub() { }
    virtual ~WebDriverClientStub() override { }

    virtual u32 magic() const override { return 4284153835; }
    virtual ByteString name() const override { return "WebDriverClient"; }

    virtual ErrorOr<OwnPtr<IPC::MessageBuffer>> handle(const IPC::Message& message) override
    {
        switch (message.message_id()) {

        case (int)Messages::WebDriverClient::MessageID::CloseSession: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::CloseSession&>(message);
            close_session();
            auto response = Messages::WebDriverClient::CloseSessionResponse { };
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SetPageLoadStrategy: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetPageLoadStrategy&>(message);
            set_page_load_strategy(request.page_load_strategy());
            return nullptr;

        }

        case (int)Messages::WebDriverClient::MessageID::SetUserPromptHandler: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetUserPromptHandler&>(message);
            set_user_prompt_handler(request.user_prompt_handler());
            return nullptr;

        }

        case (int)Messages::WebDriverClient::MessageID::SetStrictFileInteractability: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetStrictFileInteractability&>(message);
            set_strict_file_interactability(request.strict_file_interactability());
            return nullptr;

        }

        case (int)Messages::WebDriverClient::MessageID::SetIsWebdriverActive: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetIsWebdriverActive&>(message);
            set_is_webdriver_active(request.active());
            return nullptr;

        }

        case (int)Messages::WebDriverClient::MessageID::GetTimeouts: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetTimeouts&>(message);
            auto response = get_timeouts();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SetTimeouts: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetTimeouts&>(message);
            auto response = set_timeouts(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::NavigateTo: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::NavigateTo&>(message);
            auto response = navigate_to(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetCurrentUrl: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetCurrentUrl&>(message);
            auto response = get_current_url();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::Back: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::Back&>(message);
            auto response = back();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::Forward: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::Forward&>(message);
            auto response = forward();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::Refresh: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::Refresh&>(message);
            auto response = refresh();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetTitle: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetTitle&>(message);
            auto response = get_title();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetWindowHandle: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetWindowHandle&>(message);
            auto response = get_window_handle();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::CloseWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::CloseWindow&>(message);
            auto response = close_window();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SwitchToWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SwitchToWindow&>(message);
            auto response = switch_to_window(request.handle());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::NewWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::NewWindow&>(message);
            auto response = new_window(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SwitchToFrame: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SwitchToFrame&>(message);
            auto response = switch_to_frame(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SwitchToParentFrame: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SwitchToParentFrame&>(message);
            auto response = switch_to_parent_frame(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetWindowRect: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetWindowRect&>(message);
            auto response = get_window_rect();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SetWindowRect: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SetWindowRect&>(message);
            auto response = set_window_rect(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::MaximizeWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::MaximizeWindow&>(message);
            auto response = maximize_window();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::MinimizeWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::MinimizeWindow&>(message);
            auto response = minimize_window();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FullscreenWindow: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FullscreenWindow&>(message);
            auto response = fullscreen_window();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ConsumeUserActivation: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ConsumeUserActivation&>(message);
            auto response = consume_user_activation();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElement: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElement&>(message);
            auto response = find_element(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElements: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElements&>(message);
            auto response = find_elements(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElementFromElement: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElementFromElement&>(message);
            auto response = find_element_from_element(request.payload(), request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElementsFromElement: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElementsFromElement&>(message);
            auto response = find_elements_from_element(request.payload(), request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElementFromShadowRoot: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElementFromShadowRoot&>(message);
            auto response = find_element_from_shadow_root(request.payload(), request.shadow_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::FindElementsFromShadowRoot: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::FindElementsFromShadowRoot&>(message);
            auto response = find_elements_from_shadow_root(request.payload(), request.shadow_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetActiveElement: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetActiveElement&>(message);
            auto response = get_active_element();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementShadowRoot: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementShadowRoot&>(message);
            auto response = get_element_shadow_root(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::IsElementSelected: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::IsElementSelected&>(message);
            auto response = is_element_selected(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementAttribute: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementAttribute&>(message);
            auto response = get_element_attribute(request.element_id(), request.name());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementProperty: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementProperty&>(message);
            auto response = get_element_property(request.element_id(), request.name());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementCssValue: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementCssValue&>(message);
            auto response = get_element_css_value(request.element_id(), request.name());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementText: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementText&>(message);
            auto response = get_element_text(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementTagName: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementTagName&>(message);
            auto response = get_element_tag_name(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetElementRect: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetElementRect&>(message);
            auto response = get_element_rect(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::IsElementEnabled: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::IsElementEnabled&>(message);
            auto response = is_element_enabled(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetComputedRole: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetComputedRole&>(message);
            auto response = get_computed_role(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetComputedLabel: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetComputedLabel&>(message);
            auto response = get_computed_label(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ElementClick: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ElementClick&>(message);
            auto response = element_click(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ElementClear: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ElementClear&>(message);
            auto response = element_clear(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ElementSendKeys: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ElementSendKeys&>(message);
            auto response = element_send_keys(request.element_id(), request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetSource: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetSource&>(message);
            auto response = get_source();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ExecuteScript: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ExecuteScript&>(message);
            auto response = execute_script(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ExecuteAsyncScript: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ExecuteAsyncScript&>(message);
            auto response = execute_async_script(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetAllCookies: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetAllCookies&>(message);
            auto response = get_all_cookies();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetNamedCookie: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetNamedCookie&>(message);
            auto response = get_named_cookie(request.name());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::AddCookie: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::AddCookie&>(message);
            auto response = add_cookie(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::DeleteCookie: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::DeleteCookie&>(message);
            auto response = delete_cookie(request.name());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::DeleteAllCookies: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::DeleteAllCookies&>(message);
            auto response = delete_all_cookies();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::PerformActions: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::PerformActions&>(message);
            auto response = perform_actions(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::ReleaseActions: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::ReleaseActions&>(message);
            auto response = release_actions();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::DismissAlert: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::DismissAlert&>(message);
            auto response = dismiss_alert();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::AcceptAlert: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::AcceptAlert&>(message);
            auto response = accept_alert();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::GetAlertText: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::GetAlertText&>(message);
            auto response = get_alert_text();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::SendAlertText: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::SendAlertText&>(message);
            auto response = send_alert_text(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::TakeScreenshot: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::TakeScreenshot&>(message);
            auto response = take_screenshot();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::TakeElementScreenshot: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::TakeElementScreenshot&>(message);
            auto response = take_element_screenshot(request.element_id());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::PrintPage: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::PrintPage&>(message);
            auto response = print_page(request.payload());
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        case (int)Messages::WebDriverClient::MessageID::EnsureTopLevelBrowsingContextIsOpen: {

            [[maybe_unused]] auto& request = static_cast<const Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpen&>(message);
            auto response = ensure_top_level_browsing_context_is_open();
            return make<IPC::MessageBuffer>(TRY(response.encode()));

        }

        default:
            return Error::from_string_literal("Unknown message ID for WebDriverClient endpoint");
        }
    }

    virtual void close_session(
) = 0;

    virtual void set_page_load_strategy(
[[maybe_unused]] Web::WebDriver::PageLoadStrategy const& page_load_strategy) = 0;

    virtual void set_user_prompt_handler(
[[maybe_unused]] Web::WebDriver::UserPromptHandler const& user_prompt_handler) = 0;

    virtual void set_strict_file_interactability(
[[maybe_unused]] bool strict_file_interactability) = 0;

    virtual void set_is_webdriver_active(
[[maybe_unused]] bool active) = 0;

    virtual Messages::WebDriverClient::GetTimeoutsResponse get_timeouts(
) = 0;

    virtual Messages::WebDriverClient::SetTimeoutsResponse set_timeouts(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::NavigateToResponse navigate_to(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::GetCurrentUrlResponse get_current_url(
) = 0;

    virtual Messages::WebDriverClient::BackResponse back(
) = 0;

    virtual Messages::WebDriverClient::ForwardResponse forward(
) = 0;

    virtual Messages::WebDriverClient::RefreshResponse refresh(
) = 0;

    virtual Messages::WebDriverClient::GetTitleResponse get_title(
) = 0;

    virtual Messages::WebDriverClient::GetWindowHandleResponse get_window_handle(
) = 0;

    virtual Messages::WebDriverClient::CloseWindowResponse close_window(
) = 0;

    virtual Messages::WebDriverClient::SwitchToWindowResponse switch_to_window(
[[maybe_unused]] String const& handle) = 0;

    virtual Messages::WebDriverClient::NewWindowResponse new_window(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::SwitchToFrameResponse switch_to_frame(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::SwitchToParentFrameResponse switch_to_parent_frame(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::GetWindowRectResponse get_window_rect(
) = 0;

    virtual Messages::WebDriverClient::SetWindowRectResponse set_window_rect(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::MaximizeWindowResponse maximize_window(
) = 0;

    virtual Messages::WebDriverClient::MinimizeWindowResponse minimize_window(
) = 0;

    virtual Messages::WebDriverClient::FullscreenWindowResponse fullscreen_window(
) = 0;

    virtual Messages::WebDriverClient::ConsumeUserActivationResponse consume_user_activation(
) = 0;

    virtual Messages::WebDriverClient::FindElementResponse find_element(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::FindElementsResponse find_elements(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::FindElementFromElementResponse find_element_from_element(
[[maybe_unused]] JsonValue const& payload, [[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::FindElementsFromElementResponse find_elements_from_element(
[[maybe_unused]] JsonValue const& payload, [[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::FindElementFromShadowRootResponse find_element_from_shadow_root(
[[maybe_unused]] JsonValue const& payload, [[maybe_unused]] String const& shadow_id) = 0;

    virtual Messages::WebDriverClient::FindElementsFromShadowRootResponse find_elements_from_shadow_root(
[[maybe_unused]] JsonValue const& payload, [[maybe_unused]] String const& shadow_id) = 0;

    virtual Messages::WebDriverClient::GetActiveElementResponse get_active_element(
) = 0;

    virtual Messages::WebDriverClient::GetElementShadowRootResponse get_element_shadow_root(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::IsElementSelectedResponse is_element_selected(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::GetElementAttributeResponse get_element_attribute(
[[maybe_unused]] String const& element_id, [[maybe_unused]] String const& name) = 0;

    virtual Messages::WebDriverClient::GetElementPropertyResponse get_element_property(
[[maybe_unused]] String const& element_id, [[maybe_unused]] String const& name) = 0;

    virtual Messages::WebDriverClient::GetElementCssValueResponse get_element_css_value(
[[maybe_unused]] String const& element_id, [[maybe_unused]] String const& name) = 0;

    virtual Messages::WebDriverClient::GetElementTextResponse get_element_text(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::GetElementTagNameResponse get_element_tag_name(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::GetElementRectResponse get_element_rect(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::IsElementEnabledResponse is_element_enabled(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::GetComputedRoleResponse get_computed_role(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::GetComputedLabelResponse get_computed_label(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::ElementClickResponse element_click(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::ElementClearResponse element_clear(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::ElementSendKeysResponse element_send_keys(
[[maybe_unused]] String const& element_id, [[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::GetSourceResponse get_source(
) = 0;

    virtual Messages::WebDriverClient::ExecuteScriptResponse execute_script(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::ExecuteAsyncScriptResponse execute_async_script(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::GetAllCookiesResponse get_all_cookies(
) = 0;

    virtual Messages::WebDriverClient::GetNamedCookieResponse get_named_cookie(
[[maybe_unused]] String const& name) = 0;

    virtual Messages::WebDriverClient::AddCookieResponse add_cookie(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::DeleteCookieResponse delete_cookie(
[[maybe_unused]] String const& name) = 0;

    virtual Messages::WebDriverClient::DeleteAllCookiesResponse delete_all_cookies(
) = 0;

    virtual Messages::WebDriverClient::PerformActionsResponse perform_actions(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::ReleaseActionsResponse release_actions(
) = 0;

    virtual Messages::WebDriverClient::DismissAlertResponse dismiss_alert(
) = 0;

    virtual Messages::WebDriverClient::AcceptAlertResponse accept_alert(
) = 0;

    virtual Messages::WebDriverClient::GetAlertTextResponse get_alert_text(
) = 0;

    virtual Messages::WebDriverClient::SendAlertTextResponse send_alert_text(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::TakeScreenshotResponse take_screenshot(
) = 0;

    virtual Messages::WebDriverClient::TakeElementScreenshotResponse take_element_screenshot(
[[maybe_unused]] String const& element_id) = 0;

    virtual Messages::WebDriverClient::PrintPageResponse print_page(
[[maybe_unused]] JsonValue const& payload) = 0;

    virtual Messages::WebDriverClient::EnsureTopLevelBrowsingContextIsOpenResponse ensure_top_level_browsing_context_is_open(
) = 0;

private:
};

#if defined(AK_COMPILER_CLANG)
#pragma clang diagnostic pop
#endif
